##############################################################################
#
# Copyright (c) 2004 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: test_archetypes.py 606 2004-11-05 18:50:18Z sidnei $
"""

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Load fixture
import unittest
from Testing import ZopeTestCase
from OFS.Image import File

ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('DavPack')


class FileLike:

    def __init__(self, data):
        self.data = data
        self.pos = 0

    def read(self, chunk=None):
        if chunk == None:
            return self.data
        cur = self.pos
        next = cur + chunk
        self.pos = next
        return self.data[cur:next]

    def seek(self, pos, start=0):
        if start == 0:
            self.pos = pos
        if start == 1:
            self.pos += pos
        if start == 2:
            self.pos = len(self.data) - pos

    def tell(self):
        return self.pos


class FileFieldTest(ZopeTestCase.ZopeTestCase):

    def afterSetUp(self):
        from Products.Archetypes import Field
        from Products.MimetypesRegistry.MimeTypesRegistry import MimeTypesRegistry
        self.folder.mimetypes_registry = MimeTypesRegistry()
        self.field = Field.FileField('file')
        self.factory = self.field.content_class

    def test_stringio_text(self):
        from cStringIO import StringIO
        f = StringIO('x' * (1 << 19))
        f.seek(0)
        v, m, f = self.field._process_input(f, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'text/plain')
        self.failIf(f)

    def test_stringio_binary(self):
        from cStringIO import StringIO
        f = StringIO('\x00' + 'x' * (1 << 19))
        f.seek(0)
        v, m, f = self.field._process_input(f, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'application/octet-stream')
        self.failIf(f)

    def test_file_like_text(self):
        f = FileLike('x' * (1 << 19))
        f.seek(0)
        v, m, f = self.field._process_input(f, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'text/plain')
        self.failIf(f)

    def test_file_like_binary(self):
        f = FileLike('\x00' + 'x' * (1 << 19))
        f.seek(0)
        v, m, f = self.field._process_input(f, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'application/octet-stream')
        self.failIf(f)

    def test_file_upload_text(self):
        from cgi import FieldStorage
        from ZPublisher.HTTPRequest import FileUpload
        from tempfile import TemporaryFile
        fp = TemporaryFile('w+b')
        fp.write('x' * (1 << 19))
        fp.seek(0)
        env = {'REQUEST_METHOD':'PUT'}
        headers = {'content-type':'text/plain',
                   'content-length': 1 << 19,
                   'content-disposition':'attachment; filename=test.txt'}
        fs = FieldStorage(fp=fp, environ=env, headers=headers)
        f = FileUpload(fs)
        v, m, f = self.field._process_input(f, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'text/plain')
        self.assertEquals(f, 'test.txt')

    def test_file_upload_binary(self):
        from cgi import FieldStorage
        from ZPublisher.HTTPRequest import FileUpload
        from tempfile import TemporaryFile
        fp = TemporaryFile('w+b')
        fp.write('\x00' + 'x' * (1 << 19))
        fp.seek(0)
        env = {'REQUEST_METHOD':'PUT'}
        headers = {'content-type':'text/plain',
                   'content-length': 1 << 19,
                   'content-disposition':'attachment; filename=test.bin'}
        fs = FieldStorage(fp=fp, environ=env, headers=headers)
        f = FileUpload(fs)
        v, m, f = self.field._process_input(f, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'application/octet-stream')
        self.assertEquals(f, 'test.bin')

    def test_real_file_text(self):
        from tempfile import TemporaryFile
        fd = TemporaryFile('w+b')
        fd.write('x' * (1 << 19))
        fd.seek(0)
        v, m, f = self.field._process_input(fd, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'text/plain')
        self.failIf(f)

    def test_real_file_binary(self):
        from tempfile import TemporaryFile
        fd = TemporaryFile('w+b')
        fd.write('\x00' + 'x' * (1 << 19))
        fd.seek(0)
        v, m, f = self.field._process_input(fd, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'application/octet-stream')
        self.failIf(f)

    def test_real_file_force_filename_detect_mime_pdf(self):
        from tempfile import TemporaryFile
        fd = TemporaryFile('w+b')
        fd.write('\x00' + 'x' * (1 << 19))
        fd.seek(0)
        v, m, f = self.field._process_input(fd, instance=self.folder,
                                            filename='file.pdf')
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'application/pdf')
        self.assertEquals(f, 'file.pdf')

    def test_real_file_force_filename_detect_mime_xml(self):
        from tempfile import TemporaryFile
        fd = TemporaryFile('w+b')
        fd.write('\x00' + 'x' * (1 << 19))
        fd.seek(0)
        v, m, f = self.field._process_input(fd, instance=self.folder,
                                            filename='file.xml')
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'text/xml')
        self.assertEquals(f, 'file.xml')

    def test_real_file_force_mimetype(self):
        from tempfile import TemporaryFile
        fd = TemporaryFile('w+b')
        fd.write('\x00' + 'x' * (1 << 19))
        fd.seek(0)
        v, m, f = self.field._process_input(fd, instance=self.folder,
                                            mimetype='text/xml')
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'text/xml')
        self.failIf(f)

    def test_ofs_file_text(self):
        from tempfile import TemporaryFile
        fd = TemporaryFile('w+b')
        fd.write('x' * (1 << 19))
        fd.seek(0)
        f = File('f', '', '')
        self.folder._setObject('f', f)
        self.folder.f.manage_upload(fd)
        self.folder.f.content_type = 'text/plain'
        v, m, f = self.field._process_input(self.folder.f,
                                            instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        # Should retain content type.
        self.assertEquals(m, 'text/plain')
        self.assertEquals(f, self.folder.f.getId())

    def test_ofs_file_binary(self):
        from tempfile import TemporaryFile
        fd = TemporaryFile('w+b')
        fd.write('\x00' + 'x' * (1 << 19))
        fd.seek(0)
        f = File('f', '', '')
        self.folder._setObject('f', f)
        self.folder.f.manage_upload(fd)
        v, m, f = self.field._process_input(self.folder.f,
                                            instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'application/octet-stream')
        self.assertEquals(f, self.folder.f.getId())

    def test_pdata_text(self):
        from tempfile import TemporaryFile
        fd = TemporaryFile('w+b')
        fd.write('x' * (1 << 19))
        fd.seek(0)
        f = File('f', '', '')
        self.folder._setObject('f', f)
        self.folder.f.manage_upload(fd)
        v, m, f = self.field._process_input(self.folder.f.data,
                                            instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'text/plain')
        self.failIf(f)

    def test_pdata_binary(self):
        from tempfile import TemporaryFile
        fd = TemporaryFile('w+b')
        fd.write('\x00' + 'x' * (1 << 19))
        fd.seek(0)
        f = File('f', '', '')
        self.folder._setObject('f', f)
        self.folder.f.manage_upload(fd)
        v, m, f = self.field._process_input(self.folder.f.data,
                                            instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'application/octet-stream')
        self.failIf(f)

    def test_string_text(self):
        f = 'x' * (1 << 19)
        v, m, f = self.field._process_input(f, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'text/plain')
        self.failIf(f)

    def test_string_binary(self):
        f = '\x00' + 'x' * (1 << 19)
        v, m, f = self.field._process_input(f, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'application/octet-stream')
        self.failIf(f)

    def test_base_unit_text(self):
        from Products.Archetypes.BaseUnit import BaseUnit
        f = BaseUnit('f', 'x' * (1 << 19), instance=self.folder)
        v, m, f = self.field._process_input(f, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'text/plain')
        self.failIf(f)

    def test_base_unit_binary(self):
        from Products.Archetypes.BaseUnit import BaseUnit
        f = BaseUnit('f', '\x00' + 'x' * (1 << 19), instance=self.folder)
        v, m, f = self.field._process_input(f, instance=self.folder)
        self.failUnless(isinstance(v, self.factory))
        self.assertEquals(m, 'application/octet-stream')
        self.failIf(f)

def test_suite():
    suite = unittest.TestSuite()
    tests = [
        unittest.makeSuite(FileFieldTest),
        ]
    for t in tests:
        suite.addTest(t)
    return suite

if __name__ == '__main__':
    framework(descriptions=0, verbosity=1)
