# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: aliquot_list.py 759 2005-06-28 23:45:42Z sidnei $
"""

from zope.proxy import removeAllProxies
from zope.app.publisher.browser import BrowserView
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.app import zapi
from sip.interfaces.adapter import ISampleWithAliquots,\
     ISampleWithOriginAliquots
from zope.security.proxy import removeSecurityProxy
from sip.browser import api

class AliquotListing(BrowserView):
    """Aliquot Listing For Sample"""

    def listAliquotInfo(self):

        extract = self._extractInfo
        sample = removeSecurityProxy(self.context)
        adapted = ISampleWithAliquots(sample)
        aliquots = adapted.aliquots.values()
        info = map(extract, aliquots)

        return info

    def _extractInfo(self, aliquot):

        info = {}
        root = api.getRoot(self.context)
        root_url = str(zapi.getView(root, 'absolute_url', self.request))
        info['id'] = aliquot.id
        info['label'] = aliquot.label
        sample = removeAllProxies(self.context)
        info['aliquot_url'] = ('%s/projects/sip.Project.%d/aliquots/'
                               'sip.Aliquot.%d' %
                               (root_url,
                                sample.project_id,
                                aliquot.id) )
        info['url'] = ('%s/projects'
                       '/sip.Project.%d/samples/sip.Sample.%s'
                       '?aliquot_id=%s&SAMPLE_EDIT_FORM=1' %
                       (root_url,
                        sample.project_id,
                        sample.id,
                        aliquot.id))
        info['concentration'] = aliquot.getConcentrationString()
        info['weight'] = aliquot.getWeightString()
        info['volume'] = aliquot.getVolumeString()
        info['location'] = aliquot.getLocation()
        info['is_low'] = aliquot.is_low and 'Yes' or 'No'
        if aliquot.aliquot_count is None:
            info['aliquot_count'] = '&nbsp;'
        else:
            info['aliquot_count'] = aliquot.aliquot_count
        info['depleted'] = aliquot.depleted and 'Yes' or 'No'
        info['container_type'] = aliquot.getContainerType() or '&nbsp;'
        info['comment'] = aliquot.comment
        return info

    index = ViewPageTemplateFile('aliquot_list.pt')

class OriginAliquotListing(AliquotListing):

    def listAliquotInfo(self):

        extract = self._extractInfo
        sample = removeSecurityProxy(self.context)
        adapted = ISampleWithOriginAliquots(sample)
        aliquots = adapted.origin_aliquots.values()
        info = map(extract, aliquots)

        return info

    def _extractInfo(self, aliquot):

        info = super(OriginAliquotListing, self)._extractInfo(aliquot)
        aliquot = removeAllProxies(aliquot)
        sample = aliquot.sample_
        info['sample'] = sample.name
        root = api.getRoot(self.context)
        root_url = str(zapi.getView(root, 'absolute_url', self.request))
        info['sample_url'] = ('%s/projects'
                              '/sip.Project.%d/samples/sip.Sample.%d' %
                              (root_url, sample.project_id, sample.id))
        return info
