# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: sample.py 758 2005-06-28 23:02:39Z sidnei $
"""

__metaclass__ = type

from zope.interface import directlyProvides
from zope.schema import getFieldNamesInOrder

from zope.app import zapi
from zope.app.form.utility import setUpEditWidgets
from zope.app.publisher.browser import BrowserView
from zope.app.form import CustomWidgetFactory
from zope.app.form.browser import TextAreaWidget
from zope.app.form.browser import ChoiceInputWidget

from sip.browser import api
from sip.browser.admin import NextURLProxy
from sip.interfaces.utilities import ISplitDerive
from sip.interfaces.adapter import FacadeInterfaceClass
from sip.browser.widget import ROTextWidget, RODatetimeWidget, \
        ROSubjectTypeWidget


class SampleEditView(BrowserView):

    def __init__(self, context, request):
        super(SampleEditView, self).__init__(context, request)
        self.views = {}
        self.forms = {}
        sample_edit = zapi.getView(context, 'edit.html', request)
        self.views['sample'] = NextURLProxy(sample_edit, self)
        self.forms['sample'] = sample_edit.generated_form
        aliquots = zapi.getView(context, 'aliquots.html', request)
        self.views['aliquots'] = NextURLProxy(aliquots, self)
        self.forms['aliquots'] = aliquots.index
        request.form['origin'] = True
        origin_aliquots = zapi.getView(context, 'origin_aliquots.html', request)
        del request.form['origin']
        self.views['origin_aliquots'] = NextURLProxy(origin_aliquots, self)
        self.forms['origin_aliquots'] = origin_aliquots.index
        self.derived = context.derived
        source = zapi.getParent(context)
        project = zapi.getParent(source)
        aliquots = zapi.traverse(project, 'aliquots')
        adding = zapi.getView(aliquots, '+', request)
        adding = api.contained(adding, aliquots)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, aliquots)
        request.form['field.sample_id'] = context.id
        if not ('sample_type' in request or
                'sample_type_id' in request or
                'field.type_id' in request):
            request.form['sample_type_id'] = context.type_id
        request.form['SAMPLE_EDIT_FORM'] = 1

        aliquot_add = zapi.getView(adding, 'add_aliquot.html', request)
        self.views['aliquot'] = NextURLProxy(aliquot_add, self)
        self.forms['aliquot'] = aliquot_add.generated_form

        if 'aliquot_id' in request:
            aliquot_name = 'sip.Aliquot.%s' % request['aliquot_id']
            aliquot = aliquots[aliquot_name]
            aliquot_edit = zapi.getView(aliquot, 'edit.html', request)
            self.views['aliquot_edit'] = NextURLProxy(aliquot_edit, self)
            self.forms['aliquot_edit'] = aliquot_edit.generated_form


    def update(self):
        view = None
        if 'CHANGE_SAMPLE' in self.request:
            view = 'sample'
        elif 'ADD_ALIQUOT' in self.request:
            view = 'aliquot'
        elif 'EDIT_ALIQUOT' in self.request:
            view = 'aliquot_edit'
        if view:
            view = self.views[view].getView()
            if not view.update_status:
                return view.update()

    def nextURL(self):
        postfix = '/@@edit_sample.html'
        postfix += 'ADD_ALIQUOT' in self.request and '#aliquot' or ''
        postfix += 'EDIT_ALIQUOT' in self.request and '#aliquot_edit' or ''
        return (str(zapi.getView(self.context, "absolute_url", self.request))
                + postfix)


class AddSample:

    def _setUpWidgets(self):
        request = self.request
        self.technician_id_widget = CustomWidgetFactory(
            ChoiceInputWidget,
            size = 1)
        if not hasattr(self, 'type_id_widget'):
            self.type_id_widget = CustomWidgetFactory(
                ChoiceInputWidget,
                size=1,
                extra='id="type-id-chooser" onchange="getFields();"')
        self.comments_widget = CustomWidgetFactory(
            TextAreaWidget,
            width=60,
            height=8)
        res = super(AddSample, self)._setUpWidgets()
        if not 'field.type_id' in request:
            type_id = iter(self.type_id_widget.vocabulary).next().value
        else:
            type_id = self.type_id_widget.getInputValue()
        if type_id:
            # We get the schema for the given type id
            schema = api.getSchemaFor(type_id, self.context)
            # ...and set up the widgets for this schema
            setUpEditWidgets(self, schema)
            saa = []
            field_names = self.fieldNames[:]
            s_names = getFieldNamesInOrder(schema)
            # then append the field names of this schema
            # to the pre-existing ones
            field_names.extend(s_names)
            if self._set_after_add:
                # and append also to the _set_after_add
                # list, which is used by the createAndAdd
                # to figure which values will be set
                saa.extend(list(self._set_after_add))
            saa.extend(s_names)
            self.fieldNames = field_names
            self._set_after_add = saa
            # then we create a FacadeInterface
            self.facade = FacadeInterfaceClass("AugmentedSample")
            # and add both schemas to it
            self.facade.addTarget(self.schema)
            self.facade.addTarget(schema)
            self.orig_schema = self.schema
        return res

    # Now stuff gets tricky:
    #
    # Given the way update() is implemented we need to juggle around
    # interfaces because we have no easy way to override parts of
    # createAndAdd.
    #
    # Here's what happens:
    #
    # * update() is called, needs the FacadeInterface to fetch the
    #   values for all the fields
    #
    # * createAndAdd() is called inside update()
    #
    # * create() is called inside createAndAdd() it needs the original
    #   interface, to get the right adapter, and create() needs to
    #   return a wrapped object because the adapter expects the object
    #   to be wrapped
    #
    # * add() is called inside createAndAdd() it needs the
    #   FacadeInterface, because its time to set the values from the
    #   sample_type schema now.  We use directlyProvides() to bypass the
    #   getAdapter() call and return an adapter from add() instead.

    def update(self):
        self.schema = self.facade
        res = super(AddSample, self).update()
        self.schema = self.orig_schema
        return res

    def createAndAdd(self, data):
        content = super(AddSample, self).createAndAdd(data)
        self.contentName = 'sip.Sample.%s' % content.id
        return content

    def create(self, *args, **kw):
        self.schema = self.orig_schema
        project = api.queryProject(self.context)
        projectid = project.id
        kw['project_id'] = projectid
        if kw.get('other_id', None) is not None:
            kw['other_id'] = str(kw['other_id'])
        if kw.get('subject_id', None) is not None:
            kw['subject_id'] = str(kw['subject_id'])
        factory = api.factory('sip.Sample', context=self.context)
        return api.contained(factory(**kw), self.context)

    def add(self, content):
        self.schema = self.facade
        content = super(AddSample, self).add(content)
        adapted = self.orig_schema(content)
        directlyProvides(adapted, self.schema)
        return adapted

    def nextURL(self):
        request = self.request
        project = api.getProject(self.context)
        sample = zapi.traverse(project,
                               'samples/%s' % self.contentName)
        request.form['message'] = 'Sample Added'
        return "%s#aliquot" % zapi.getView(sample, 'absolute_url', request)

class AddDerivedSample(AddSample):

    def create(self, *args, **kw):
        aliquot_ids = self.request['aliquot_ids']
        kw['derived'] = 1
        sample = super(AddDerivedSample, self).create(*args, **kw)
        ds = zapi.getUtility(ISplitDerive, context=self.context)
        ds.derive(sample.id, aliquot_ids)
        return sample

    def _setUpWidgets(self):
        request = self.request
        if request.get('collection_date'):
            collection_date = request['collection_date']
            self.collection_date_widget = CustomWidgetFactory(
                RODatetimeWidget,
                value=collection_date)
        if request.get('subject_type_id'):
            subject_type_id = request['subject_type_id']
            self.subject_type_id_widget = CustomWidgetFactory(
                ROSubjectTypeWidget,
                _data=subject_type_id)
        if request.get('subject_id'):
            subject_id = request['subject_id']
            self.subject_id_widget = CustomWidgetFactory(
                ROTextWidget,
                value=subject_id)
        if request.get('other_id'):
            other_id = request['other_id']
            self.other_id_widget = CustomWidgetFactory(
                ROTextWidget,
                value=other_id)
        self.type_id_widget = CustomWidgetFactory(
            ChoiceInputWidget,
            extra='id="type-id-chooser" onchange="getFields();"',
            size=1)
        self.comments_widget = CustomWidgetFactory(
            TextAreaWidget,
            width=60,
            height=8)

        return super(AddDerivedSample, self)._setUpWidgets()

class EditSample:

    def _setUpWidgets(self):
        request = self.request
        self.type_id_widget = CustomWidgetFactory(
            ChoiceInputWidget,
            size=1,
            extra=('id="type-id-chooser" '
                   'onchange="getFields();"'))
        self.comments_widget = CustomWidgetFactory(TextAreaWidget,
                                            width=60,
                                            height=8)
        res = super(EditSample, self)._setUpWidgets()
        if not 'field.type_id' in request:
            type_id = self.context.type_id
        else:
            type_id = self.type_id_widget.getInputValue()
            # If there's a type_id in the request, then
            # we must reset the adapter so it gets
            # the right schema.
            self.adapted._reset(type_id)
        if type_id:
            # We get the schema for the given type id
            schema = api.getSchemaFor(type_id, self.context)
            # ...and set up the widgets for this schema
            setUpEditWidgets(self, schema, context=self.adapted,
                             source=self.adapted)
            field_names = self.fieldNames[:]
            s_names = getFieldNamesInOrder(schema)
            # then append the field names of this schema
            # to the pre-existing ones
            field_names.extend(s_names)
            self.fieldNames = field_names
            # then we create a FacadeInterface
            self.facade = FacadeInterfaceClass("AugmentedSample")
            # and add both schemas to it
            self.facade.addTarget(self.schema)
            self.facade.addTarget(schema)
            self.schema = self.facade
        return res

    def update(self):
        self.update_status = super(EditSample, self).update()
        return self.update_status
