# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: schema.py 758 2005-06-28 23:02:39Z sidnei $
"""

__metaclass__ = type

from persistent import Persistent

from zope.interface import implements, directlyProvides, directlyProvidedBy
from zope.schema.interfaces import ValidationError, ConstraintNotSatisfied
from zope.schema import getFieldNamesInOrder
from zope.proxy import removeAllProxies
from zope.security.proxy import removeSecurityProxy

from zope.app import zapi
from zope.app.location import LocationProxy
from zope.app.form.browser.editview import EditView
from zope.app.publisher.browser import BrowserView
from zope.app.pagetemplate.engine import Engine
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from sip.schema.interfaces import IMutableSchema

from sip.i18n import _
from sip.browser.admin import NextURLProxy
from sip.browser import api
from sip.schema.browser import EditSchema
from sip.interfaces.fields import ITALESConstrainedField
from sip.interfaces.sample import ISampleSchema
from zope.component.interfaces import IFactory

class SchemaEditView(EditSchema):

    def __init__(self, context, request):
        context = IMutableSchema(context)
        super(SchemaEditView, self).__init__(context, request)
        adding = zapi.getView(self.context, 'addfield.html', request)
        adding = NextURLProxy(adding, self)
        adding = api.contained(adding, self.context)
        self.views = {}
        self.forms = {}
        self.views['schema'] = adding
        self.forms['schema'] = adding.generated_form

    def update(self):
        view = None
        request = self.request
        if 'ADD_FIELD' in request:
            view = 'schema'
        elif 'DELETE' in request:
            if self.update_status is None:
                self.update_status = super(SchemaEditView, self).update()
                return self.update_status
        elif ('MOVE_UP' in request or
              'MOVE_DOWN' in request):
            if self.update_status is None:
                self.update_status = super(SchemaEditView, self).update()
                return self.update_status
        if view:
            view = self.views[view].getView()
            if self.update_status is None:
                self.update_status = view.update()
                return self.update_status

    edit = ViewPageTemplateFile('editschema.pt')

    def nextURL(self):
        return (str(zapi.getView(self.context, "absolute_url", self.request))
                + '/@@editschema.html')

class SchemaAddField:

    def add(self, content):
        adding = zapi.getView(self.context, '+', self.request)
        adding.contentName = self.contentName
        content = removeAllProxies(content)
        return adding.add(content)

    def create(self, *args, **kw):
        if not hasattr(self, 'contentName') or not self.contentName:
            self.contentName = kw['id']
        del kw['id']
        if self.contentName in getFieldNamesInOrder(ISampleSchema):
            raise ConstraintNotSatisfied, _('Schema field name cannot '
                                            'have the same id as '
                                            'fields from Sample.')
        factory = zapi.getUtility(IFactory, kw['type'], context=self)
        del kw['type']
        usetales = kw.get('usetales')
        del kw['usetales']
        if usetales:
            kw['constraint'] = TALESConstraint()
        field = factory(*args, **kw)
        if usetales:
            directlyProvides(field, directlyProvidedBy(field),
                             ITALESConstrainedField)
        return field

    def nextURL(self):
        return (str(zapi.getView(self.context, "absolute_url", self.request))
                + '/@@editschema.html')

class SampleTypeSchema(BrowserView):

    def index(self):
        url = str(zapi.getView(self.context, "absolute_url", self.request))
        url += '/schema/@@editschema.html'
        return self.request.response.redirect(url)

class EditConstrainedField(EditView):

    def __init__(self, context, request):
        # XXX This is bad. We need to figure out why
        # we are getting Unauthorized here even if we
        # **have** the right permission.
        context = removeSecurityProxy(context)
        context = LocationProxy(context.constraint,
                                context,
                                name='constraint')
        super(EditConstrainedField, self).__init__(context, request)


class TALESConstraint(Persistent):

    implements(ITALESConstrainedField)

    _expression = None

    def __init__(self, expression=u'python: 1'):
        self.expression = expression

    def __call__(self, value):
        expression = self.compiled_expression()
        if expression is not None:
            valid = expression(
                Engine.getContext(nothing=None,
                                  value=value))
            if not valid:
                raise ValidationError(ConstraintNotSatisfied,
                                      value, expression)
            return valid

    def compiled_expression(self):
        # We should store a compiled version, but the bastard is not
        # picklable!
        expression = self.expression
        if expression is not None:
            expression = Engine.compile(expression)
        return expression

    def get_expression(self):
        return super(TALESConstraint, self).__getattribute__('_expression')

    def set_expression(self, value):
        assert not value.startswith('string'), _('We only accept python: '
                                                 'TALES expressions')
        if not value.startswith('python:'):
            value = "python: %s" % value
        super(TALESConstraint, self).__setattr__('_expression', value)

    expression = property(get_expression, set_expression)
