# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: traversal.py 758 2005-06-28 23:02:39Z sidnei $
"""

__metaclass__ = type

from zope.interface import implements
from zope.component import queryView
from zope.exceptions import NotFoundError
from zope.proxy import getProxiedObject as securityProxiedObject

from zope.publisher.interfaces import IPublishTraverse
from zope.app.traversing.interfaces import ITraversable
from zope.app.container.interfaces import IReadContainer
from zope.app.location import LocationProxy
from zope.app import zapi
from zope.app.container._zope_app_container_contained import getProxiedObject \
     as containerProxiedObject

from sip.schema.interfaces import IMutableSchema
from sip.browser import api
from sip.interfaces.sampletype import ISampleType
from sip.interfaces.project import IProject

_marker = object()

class Traverser:

    implements(IPublishTraverse)

    def __init__(self, context, request=None):
        self.context = context
        self.request = request

    def publishTraverse(self, request, name):
        ob = self.context
        view = queryView(ob, name, request)
        if view is not None:
            return view
        adapter = ITraversable(self.context, None)
        if adapter is None:
            raise NotFoundError, name
        return adapter.traverse(name, None)

    def browserDefault(self, request):
        ob = self.context
        view_name = zapi.getDefaultViewName(ob, request)
        return ob, (view_name,)

class BaseTraversable:

    implements(ITraversable)

    def __init__(self, context):
        self._context = context

    def traverse(self, name, furtherPath):
        context = self._context

        subob = getattr(context, name, _marker)

        if subob is _marker:
            context = IReadContainer(context, context)
            try:
                subob = context[name]
            except KeyError:
                raise NotFoundError, name

        return subob


class SampleTypeTraversable(BaseTraversable):

    __used_for__ = ISampleType

    def traverse(self, name, furtherPath):
        return super(SampleTypeTraversable, self).traverse(name, furtherPath)

class SchemaTraversable(BaseTraversable):

    __used_for__ = IMutableSchema

    def traverse(self, name, furtherPath):
        adapter = IMutableSchema(self._context)
        subob = adapter.get(name, _marker)

        if subob is not _marker:
            # Don't change. Order matters here.  If you get the
            # contained object first and then you get the security
            # proxied object, the container proxy doesn't goes away.
            subob = securityProxiedObject(subob)
            subob = containerProxiedObject(subob)
            subob = LocationProxy(subob, self._context, name)
            return subob

        return super(SchemaTraversable, self).traverse(name, furtherPath)

class LocationConstraint:

    def __init__(self, context):
        self.context = context

    def __call__(self, value):
        name, obj = value
        id = name.split('.')[-1]
        return int(id) in self.context.valid_locations

class TechnicianConstraint:

    def __init__(self, context):
        self.context = context

    def __call__(self, value):
        name, obj = value
        id = name.split('.')[-1]
        return int(id) in self.context.valid_technicians

class SampleTypeConstraint:

    def __init__(self, context):
        self.context = context

    def __call__(self, value):
        name, obj = value
        id = name.split('.')[-1]
        return int(id) in self.context.valid_sample_types

class ProjectTraversable(BaseTraversable):

    __used_for__ = IProject

    def traverse(self, name, furtherPath):
        subob = _marker

        if name == 'locations':
            from sip.source.location import Location
            subob = Location(
                context=self._context,
                use_project=False,
                constraint=LocationConstraint(self._context)
                )
            subob = api.contained(subob, self._context, name=name)
            return subob

        if name == 'technicians':
            from sip.source.technician import Technician
            subob = Technician(
                context=self._context,
                use_project=False,
                constraint=TechnicianConstraint(self._context)
                )
            subob = api.contained(subob, self._context, name=name)
            return subob

        if name == 'sampletypes':
            from sip.source.sampletype import SampleType
            subob = SampleType(
                context=self._context,
                use_project=False,
                constraint=SampleTypeConstraint(self._context)
                )
            subob = api.contained(subob, self._context, name=name)
            return subob

        if subob is not _marker:
            return subob

        return super(ProjectTraversable, self).traverse(name, furtherPath)
