# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

#!/usr/bin/env python
"""
$Id: importer.py 749 2005-06-28 22:21:49Z sidnei $
"""

import sys
import code
from os import curdir
from os.path import join, abspath, dirname
from optparse import OptionParser
from sip import __all__ as sip_types
from sip import *

NULL = 'NULL'
NEW_NULL = None
connection = None
db_type = None
order = []

try:
    __file__
except NameError:
    # Called directly, so no __file__ global exists.
    _prefix = abspath(curdir)
else:
    # Called by another script.
    _prefix = abspath(dirname(__file__))

class Wrapper:
    """A Mixin Class"""

    def __init__(self, connection):
        from sqlobject.dbconnection import DBAPI
        from sqlos.transaction import SQLObjectTransactionManager
        from transaction import get_transaction

        DBAPI.__init__(self)
        self._connection = connection.makeConnection()
        self.debug = connection.debug
        self.autoCommit = None
        self.supportTransactions = None
        self._dm = SQLObjectTransactionManager()
        get_transaction().join(self._dm)

    def makeConnection(self):
        return self._connection

    def _runWithConnection(self, meth, *args):
        from zope.app.rdb.interfaces import DatabaseException

        conn = self.getConnection()
        try:
            try:
                val = meth(conn, *args)
            except Exception, exc:
                raise DatabaseException, tuple(exc.args)
        finally:
            self.releaseConnection(conn)
        return val

seq_update = ("select setval('%(table_name)s_%(table_name)s_id_seq', "
             "max(%(table_name)s_id)) from %(table_name)s;")

def main(order, stream=sys.stdout):
    global connection
    from transaction import get_transaction
    from utils import csv_read_dir, Exc

    maxWidth = 80
    try:
        import curses
    except ImportError:
        pass
    else:
        curses.setupterm()
        maxWidth = curses.tigetnum('cols')
    maxWidth -= len(('xxxxxx/xxxxxx xxxxxx/xxxxxx '
                     '(xxx.x%) Err: xxxxxx Curr: ')) + 1
    transaction = get_transaction()
    tables = csv_read_dir(join(_prefix, 'processed'))
    exceptions = Exc()
    total = 0
    count = 0
    err_count = 0
    lastWidth = 0
    for tname in order:
        table = tables[tname]
        total += len(table)
    for tname in order:
        klass = table_map[tname]
        table = tables[tname]
        table_len = len(table)
        table_count = 0
        exceptions[tname] = []
        for row in table:
            for k, v in row.items():
                if v == NULL:
                    row[k] = NEW_NULL
                elif k == 'exponent':
                    row[k] = eval(v)
            try:
                stream.write("\r%6d" % (table_count + 1))
                stream.write("/%6d " % table_len)
                perc = (count + 1) * 100.0 / total
                stream.write("%6d" % (count + 1))
                stream.write("/%6d " % total)
                stream.write("(%5.1f%%) " % perc)
                stream.write("Err: %6d " % err_count)
                name = repr(tname[:maxWidth])
                width = len(name)
                if width < lastWidth:
                    name += " " * (lastWidth - width)
                lastWidth = width
                count += 1
                table_count += 1
                stream.write("Curr: %s" % name)
                stream.flush()
                klass(**row)
                transaction.commit(1)
            except Exception, msg:
                err_count += 1
                exceptions[tname].append((row, msg))
                transaction.abort(1)
        if db_type == 'postgres':
            query = seq_update % {'table_name':tname}
            connection.query(query)
    exceptions.printNumbers()
    local = locals()
    local.update(globals())
    code.interact(local=local)
    transaction.commit()

def interact():
    local = locals()
    local.update(globals())
    code.interact(local=local)
    return True

def initialize(connection):
    global order, table_map
    glb = globals()
    for t in sip_types:
        glb[t]._connection = connection
        glb[t].createTable(ifNotExists=1)
    del glb
    table_map = {'project':Project,
                 'sample':Sample,
                 'additional_sample_data':AdditionalSampleData,
                 'sample_type':SampleType,
                 'sample_type_project':SampleTypeProject,
                 'container':Container,
                 'container_location':ContainerLocation,
                 'container_type':ContainerType,
                 'location':Location,
                 'location_project':LocationProject,
                 'aliquot':Aliquot,
                 'technician':Technician,
                 'technician_project':TechnicianProject,
                 'principal':Principal,
                 'concentration_unit':ConcentrationUnit,
                 'volume_unit':VolumeUnit,
                 'weight_unit':WeightUnit,
                 'sample_type_schema':SampleTypeSchema,
                 'schema_field':SchemaField
                 }
    for kname in sip_types:
        for k, v in table_map.items():
            if v.__name__ == kname:
                order.append(k)

def bootstrap():
    st_data = ({'id':1, 'name':'animal'},
               {'id':2, 'name':'other'},
               {'id':3, 'name':'ego'})
    ct_data = ({'id':1,
                'name':'Root Container',
                'commit_message':'Root Container',
                'parent_id':None},)
    SubjectType.clearTable()
    for s in st_data:
        SubjectType(**s)
    Container.clearTable()
    for c in ct_data:
        Container(**c)

def reset():
    t = [table_map.get(o) for o in order]
    t.reverse()
    [o.clearTable() for o in t]
    bootstrap()
    o._connection._connection.commit()

def consume(options, args):
    global connection, db_type, order
    from pprint import pprint

    user = getattr(options, 'username') or 'zope3'
    passwd = getattr(options, 'password') or '123'
    database = getattr(options, 'database') or 'zope3'
    db_type = getattr(options, 'db_type') or  'postgres'
    host = getattr(options, 'host') or 'localhost'
    debug = getattr(options, 'debug') or False

    print 'Accessing with:'
    pprint({'user':user,
            'password':passwd,
            'database':database,
            'db_type':db_type,
            'host':host
            })

    if db_type == 'postgres':
        from sqlobject.postgres.pgconnection import PostgresConnection
        Connection = PostgresConnection
    if db_type == 'sybase':
        from sqlobject.sybase.sybaseconnection import SybaseConnection
        Connection = SybaseConnection

    class ConnectionWrapper(Wrapper, Connection): pass

    conn = Connection(db=database, user=user, passwd=passwd,
                      host=host, debug=debug)

    connection = ConnectionWrapper(conn)

    initialize(connection)

    if args:
        order = filter(lambda x: x in args, order)

    if db_type == 'postgres':
        connection.query("SET DateStyle TO 'NonEuropean';")

    if getattr(options, 'interact', None) is not None:
        return interact()

    return main(order)

if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option('-i', '--interact',
                      action='store_const',
                      const=True,
                      dest='interact',
                      help='Enter Interactive Mode')
    parser.add_option('-H', '--host',
                      dest='host',
                      help='Host to connect to')
    parser.add_option('-d', '--database',
                      dest='database',
                      help='Database to connect to')
    parser.add_option('-u', '--username',
                      dest='username',
                      help='Username to use for connection')
    parser.add_option('-p', '--password',
                      dest='password',
                      help='Password to use for connection')
    parser.add_option('-t', '--type',
                      dest='db_type',
                      help='Connection Type')
    parser.add_option('-D', '--debug',
                      action='store_const',
                      const=True,
                      dest='debug',
                      help='Debug queries')
    (options, args) = parser.parse_args()
    consume(options, args)
