# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: __init__.py 749 2005-06-28 22:21:49Z sidnei $
"""

__metaclass__ = type

from datetime import date
from sip.interfaces.csv import ILookupHelper
from sip.browser import api
from sqlobject.main import SQLObjectNotFound
from zope.interface import implements
from zope.app.datetimeutils import parse
from zope.app.container.interfaces import IReadContainer

class LookupHelper:

    implements(ILookupHelper)

    def __init__(self, name, func):
        self.name = name
        self.func = func

    def newName(self):
        return self.name

    def lookup(self, context, value):
        return self.func(context, value)

def getIdByField(context, factory, field, value):
    klass = api.factory(factory, context=context)
    res = klass.select('%s = %r' % (field, value))
    if res.count():
        return iter(res).next().id
    raise ValueError, "Couldn't lookup value %r" % value

def getProjectId(context, value):
    project = api.queryProject(context)
    if project is not None and project.name == value:
        return project.id
    return getIdByField(context, 'sip.Project', 'name', value)

def getSampleId(context, value):
    return getIdByField(context, 'sip.Sample', 'name', value)

def getSampleName(context, value):
    # Just return the original value
    return value

def getSampleTypeId(context, value):
    return getIdByField(context, 'sip.SampleType', 'name', value)

def getWeightUnitId(context, value):
    try:
        return getIdByField(context, 'sip.WeightUnit', 'label', value)
    except ValueError:
        return None

def getVolumeUnitId(context, value):
    try:
        return getIdByField(context, 'sip.VolumeUnit', 'label', value)
    except ValueError:
        return None

def getConcentrationUnitId(context, value):
    try:
        return getIdByField(context, 'sip.ConcentrationUnit', 'label', value)
    except ValueError:
        return None

def getTechnicianId(context, value):
    try:
        return getIdByField(context, 'sip.Technician', 'username', value)
    except ValueError:
        return None

def getContainerTypeId(context, value):
    try:
        return getIdByField(context, 'sip.ContainerType', 'name', value)
    except ValueError:
        return None

def getShipment(context, name):
    klass = api.factory('sip.Shipment', context=context)
    res = klass.select('name = %r' % name)
    if not res.count():
        raise ValueError, "Couldn't lookup shipment %r" % name
    shipment = res[0]
    shipments = IReadContainer(api.getProject(context))['shipments']
    shipment = shipments['outgoing']['incomplete']['sip.Shipment.%s' %
                                                   shipment.id]
    return shipment

def getLocation(context, name):
    klass = api.factory('sip.Location', context=context)
    res = klass.select('name = %r' % name)
    if not res.count():
        raise ValueError, "Couldn't lookup location %r" % name
    location = res[0]
    location = api.getRoot(context)['locations']['sip.Location.%s'
                                                 % location.id]
    return location

def getContainerId(context, value):
    if value.startswith('path://'):
        value = value[len('path://'):]
    if not value:
        return None
    value = [s.strip() for s in value.split('/')]
    start = 1
    if value[0] == 'shipment':
        root = getShipment(context, value[1])
        start = 2
    else:
        root = getLocation(context, value[0])
    container = root.getRootContainer()
    klass = api.factory('sip.Container', context=context)
    for name in value[start:]:
        if not name:
            continue
        res = klass.select(('name = %s and parent_id = %d ' %
                            (klass.sqlrepr(name),
                             container.id)))
        if not res.count():
            raise ValueError, "Couldn't lookup container %r in path %r" % (
                name, '/'.join(value))
        container = res[0]
    return container.id

def getSubjectTypeId(context, value):
    value = value.lower().strip()
    try:
        return getIdByField(context, 'sip.SubjectType', 'name', value)
    except ValueError:
        return None

def cleanSubjectType(context, value):
    return value.lower().strip()

def getIntegerOrNone(context, value):
    return value and int(value) or None

def getFloatOrNone(context, value):
    return value and float(value) or None

def getBooleanOrFalse(context, value):
    if value in ('True', 'TRUE', 'true', 't', 'T', '1', 1):
        return True
    return False

def getDateOrNone(context, value):
    if not value:
        return None
    parts = parse(value)
    return date(parts[0], parts[1], parts[2])

SampleId = LookupHelper('sample_id', getSampleId)
SampleName = LookupHelper('name', getSampleName)
SampleTypeId = LookupHelper('type_id', getSampleTypeId)
WeightUnitId = LookupHelper('weight_id', getWeightUnitId)
VolumeUnitId = LookupHelper('volume_id', getVolumeUnitId)
TechnicianId = LookupHelper('technician_id', getTechnicianId)
ConcentrationUnitId = LookupHelper('concentration_id', getConcentrationUnitId)
ContainerId = LookupHelper('container_id', getContainerId)
SubjectType = LookupHelper('subject_type', cleanSubjectType)
SubjectTypeId = LookupHelper('subject_type_id', getSubjectTypeId)
ChicletCount = LookupHelper('aliquot_count', getIntegerOrNone)
IsLow = LookupHelper('is_low', getBooleanOrFalse)
Depleted = LookupHelper('depleted', getBooleanOrFalse)
Volume = LookupHelper('volume', getFloatOrNone)
Weight = LookupHelper('weight', getFloatOrNone)
Concentration = LookupHelper('concentration', getFloatOrNone)
ProjectId = LookupHelper('project_id', getProjectId)
ContainerTypeId = LookupHelper('type_id', getContainerTypeId)
CollectionDate = LookupHelper('collection_date', getDateOrNone)
