# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

#! /usr/bin/python2.3
"""
$Id: massager.py 749 2005-06-28 22:21:49Z sidnei $
"""

import csv
from os import curdir
from os.path import join, abspath, dirname
from utils import csv_read_dir, Table

TRUE = 't'
FALSE = 'NULL'
NULL = 'NULL'
START_DATE = '01-01-1970'
COMMIT_MESSAGE = 'Imported from CSV'
NEW_NULL = ''

try:
    __file__
except NameError:
    # Called directly, so no __file__ global exists.
    _prefix = abspath(curdir)
else:
    # Called by another script.
    _prefix = abspath(dirname(__file__))

isnum = [str(s) for s in range(0, 10)] + ['+', '-']

def getNum(astring):
    f = lambda x: x in isnum
    return filter(f, astring)

def parseType(astring):
    parts = astring.split()
    maybe = parts[0]
    try:
        temp = int(getNum(maybe))
    except ValueError:
        return NULL, astring
    if parts[1].upper() == 'C':
        del parts[1]
    return temp, ' '.join(parts[1:])

class Writer:

    def __init__(self, tables, order=None):
        self.tables = tables
        self.order = order
        self.out = {}

    def crunch(self):
        if not self.order:
            order = self.tables.keys()
            order.sort()
            self.order = order
        for table in self.order:
            method = getattr(self, 'do_%s' % table, None)
            if method is not None:
                method()
            else:
                print "%s not done yet" % table
        self.write()

    def write(self):
        for k, v in self.out.items():
            print 'Writing %s' % k
            out = join(_prefix, 'processed', '%s.csv' % k)
            outfile = open(out, 'w')
            writer = csv.writer(outfile)
            writer.writerow(v.keys())
            writer.writerows(v.values())
            outfile.close()

    def do_valid_sample_types(self):
        st = self.tables['valid_sample_types'].copy()
        req = self.tables['required_fields'].copy()
        self.sfids = sfids = {}
        self.out['schema_field'] = sts = Table('schema_field',
                                               ['temp_id',
                                                'schema_id',
                                                'name',
                                                'title',
                                                'description',
                                                'identifier',
                                                'required'])
        self.out['sample_type_schema'] = sch = Table('sample_type_schema',
                                                     ['project_id',
                                                      'sample_type_id'])
        self.out['sample_type_project'] = stp = Table('sample_type_project',
                                                      ['project_id',
                                                       'sample_type_id'])
        st.rename('sample_type_id', 'id')
        st.rename('owner_id', 'project_id')
        st.rename('sample_type_desc', 'name')
        st['commit_message'] = COMMIT_MESSAGE
        identifiers = ['identifier%s_desc' % i for i in range(1,4)]
        for i, r in enumerate(st):
            stp.append([r['project_id'], r['id']])
            schema_id = (sch.append([r['project_id'], r['id']]) + 1)
            for idx, name in enumerate(identifiers):
                if r[name] == NULL: continue
                ident = "identifier%s" % (idx+1)
                tmp_id = "%s-%s" % (r['id'], ident)
                sf_id = sts.append([tmp_id, schema_id,
                                   r[name].lower().replace(' ', '_'),
                                   r[name], r[name], ident,
                                   FALSE]) + 1
                sfids[tmp_id] = sf_id
        for i, r in enumerate(req):
            temp_id = "%s-%s" % (r['sample_type_id'],
                                 r['column_name'])
            res = sts.select(temp_id=temp_id)
            sts.update(res[0] - 1, 'required', TRUE)
        for ident in identifiers:
            del st[ident]
        # Now, unique-fy the sample types by name
        st_unique = st.copy(empty=True)
        st_name = {}
        self._st_map = st_map = {}
        for i, r in enumerate(st):
            st_id = st_name.get(r['name'])
            if st_id is not None:
                st_map[r['id']] = st_id
                res = sch.select(sample_type_id=r['id'])
                sch.update(res[0] - 1, 'sample_type_id', st_id)
                res = stp.select(sample_type_id=r['id'])
                stp.update(res[0] - 1, 'sample_type_id', st_id)
                continue
            st_name[r['name']] = r['id']
            values = []
            for k in st.keys():
                values.append(r[k])
            st_unique.append(values)
        del st_unique['project_id']
        sts['type'] = 'zope.schema._bootstrapfields.TextLine'
        del sts['identifier']
        self.out['sample_type'] = st_unique

    def do_weight_units(self):
        wu = self.tables['weight_units'].copy()
        self.out['weight_unit'] = wu
        wu['id'] = '%(id)s'
        wu['start_date'] = START_DATE
        wu['end_date'] = NULL
        wu['commit_message'] = COMMIT_MESSAGE

    def do_volume_units(self):
        vu = self.tables['volume_units'].copy()
        self.out['volume_unit'] = vu
        vu['id'] = '%(id)s'
        vu['start_date'] = START_DATE
        vu['end_date'] = NULL
        vu['commit_message'] = COMMIT_MESSAGE

    def do_concentration_units(self):
        cu = self.tables['concentration_units'].copy()
        self.out['concentration_unit'] = cu
        cu['id'] = '%(id)s'
        cu['exponent'] = 1
        cu['start_date'] = START_DATE
        cu['end_date'] = NULL
        cu['commit_message'] = COMMIT_MESSAGE

    def do_valid_technicians(self):
        techs = self.tables['valid_technicians'].copy()
        self.out['technician'] = techs
        tp = Table('technician_project', ['project_id',
                                          'technician_id'])
        self.out['technician_project'] = tp
        self._techs = {}
        for row in techs:
            tp.append([row['owner_id'], row['tid']])
            self._techs[row['technician_id']] = row['tid']
        techs.rename('technician_description', 'fullname')
        techs.rename('technician_id', 'username')
        techs.rename('tid', 'id')
        del techs['owner_id']
        techs['commit_message'] = COMMIT_MESSAGE

    def do_valid_users(self):
        principals = self.tables['valid_users'].copy()
        self.out['principal'] = principals
        principals.rename('user_id', 'username')
        del principals['owner_id']
        del principals['start_date']
        del principals['end_date']
        principals['id'] = '%(id)s'
        principals['password'] = 'user%(id)s'
        principals['fullname'] = ''
        for index, row in enumerate(principals):
            principals.update(index, 'fullname', row['username'])

    def do_valid_owners(self):
        projects = self.tables['valid_owners'].copy()
        self.out['project'] = projects
        projects.rename('owner_id', 'id')
        projects.rename('owner_desc', 'name')
        projects['require_commit_message'] = FALSE
        projects['commit_message'] = COMMIT_MESSAGE

    def do_valid_locations(self):
        locations = self.tables['valid_locations'].copy()
        self.out['location'] = locations
        container_location = Table('container_location',
                                   ['location_id',
                                    'container_id'])
        self.out['container_location'] = container_location
        containers = Table('container', ['parent_id',
                                          'type_id'])
        self.out['container'] = containers
        c_types = Table('container_type', ['name',
                                            'root'])
        self.out['container_type'] = c_types
        lp = Table('location_project', ['project_id',
                                        'location_id'])
        self.out['location_project'] = lp
        locations['temperature'] = 0
        locations['root_type_id'] = None
        locations['locator1_type_id'] = None
        locations['locator2_type_id'] = None
        locations['locator3_type_id'] = None
        containers['name'] = 'Container %(id)s'
        self.root_cid = {}
        for index, row in enumerate(locations):
            lid = row['location_id']
            lp.append([row['owner_id'], lid])
            lt = row['location_type']
            temp, type = parseType(lt)
            locations.update(index, 'temperature', temp)
            type = ((type == NULL or type == '') and
                    lt or type)
            res = c_types.select(name=type)
            type_id = (res and res[1]['root'] == TRUE
                       and res[0] or (c_types.append([type, TRUE]) + 1))
            c_name = row['location_name']
            cid = containers.append([1, type_id, c_name]) + 2
            container_location.append([lid, cid])
            self.root_cid[lid] = cid
            locations.update(index, 'root_type_id', type_id)
            for k in ['locator1_desc', 'locator2_desc', 'locator3_desc']:
                type = row[k]
                if (type == NULL or type == ''):
                    continue
                res = c_types.select(name=type)
                type_id = (res and res[1]['root'] == FALSE
                           and res[0] or (c_types.append([type, FALSE]) + 1))
                locations.update(index, '%s_type_id' % k[:-5], type_id)

        c_types['id'] = '%(id)s'
        c_types['start_date'] = START_DATE
        c_types['end_date'] = NULL
        c_types['commit_message'] = COMMIT_MESSAGE

        locations.rename('location_id', 'id')
        locations.rename('location_name', 'name')
        locations['commit_message'] = COMMIT_MESSAGE
        del locations['owner_id']
        del locations['location_type']

    def do_aliquot(self):
        aliquots = self.tables['aliquot'].copy()
        self.out['aliquot'] = aliquots
        for k in aliquots.keys():
            if k.endswith('disposition'):
                del aliquots[k]
        for i, k in enumerate(aliquots.keys()):
            if k.endswith('_unit'):
                nk = k[:-5]
                t = self.tables['%s_units' % nk]
                for row in aliquots.values():
                    if row[i] == NULL: continue
                    res = t.select(label=row[i])
                    if res is None:
                        raise KeyError, "%s %s %s" % (t.name(),
                                                      k,
                                                      row[i])
                    row[i] = res[0]
        aliquots.rename('aliquot_date', 'date')
        aliquots['commit_message'] = COMMIT_MESSAGE
        aliquots['depleted'] = FALSE
        aliquots['is_low'] = FALSE
        for k in aliquots.keys():
            if k.endswith('_unit'):
                nk = k[:-5] + '_id'
                aliquots.rename(k, nk)
        aliquots['container_id'] = NULL
        aliquots['project_id'] = NULL
        aliquots.rename('aliquot_id', 'id')
        samples = self.out['sample']
        for index, row in enumerate(aliquots):
            c_id = self.alid.get(row['id'], NULL)
            aliquots.update(index, 'container_id', c_id)
            proj_id = self.proj_id.get(row['sample_id'], NULL)
            aliquots.update(index, 'project_id', proj_id)

    def do_sample(self):
        samples = self.tables['sample'].copy()
        sfids = self.sfids
        self.proj_id = {}
        self.out['sample'] = samples
        self.out['additional_sample_data'] = ad = Table(
            'additional_sample_data',
            ['sample_id', 'schema_field_id', 'value'])
        samples.rename('sample_id', 'id')
        samples.rename('sample_type_id', 'type_id')
        samples.rename('sample_name', 'name')
        samples.rename('previous_name', 'other_id')
        samples.rename('date_gener', 'date_generated')
        samples.rename('owner_id', 'project_id')
        samples['commit_message'] = COMMIT_MESSAGE
        samples['collection_date'] = NULL
        samples['subject_type_id'] = 1
        del samples['aliquot_origin_id']
        del samples['animal_id']
        del samples['ego_id']
        tech = self.out['technician']
        sf = self.out['schema_field']
        st_map = self._st_map
        for index, row in enumerate(samples):
            tech_id = self._techs.get(row['technician_id'], NULL)
            samples.update(index, 'technician_id', tech_id)
            samples.update(index, 'collection_date',
                           row['date_generated'] or START_DATE)
            self.proj_id[row['id']] = row['project_id']
            if row['name'] == NULL:
                samples.update(index, 'name', 'Sample %s' % row['id'])
            for k in ('identifier1', 'identifier2', 'identifier3'):
                temp_id = "%s-%s" % (row['type_id'], k)
                res = sf.select(temp_id=temp_id)
                if res is None: continue
                ad.append([row['id'], sfids[temp_id], row[k]])
            st_id = st_map.get(row['type_id'])
            if st_id is not None:
                samples.update(index, 'type_id', st_id)
        del sf['temp_id']
        for k in ('identifier1', 'identifier2', 'identifier3'):
            del samples[k]

    def do_location(self):
        old_loc = self.tables['location']
        new_loc = self.out['location']
        containers = self.out['container']
        container_location = self.out['container_location']
        c_types = self.out['container_type']
        self.alid = alid = {}
        self.ctid = ctid = {}
        for r in old_loc:
            lid = r['location_id']
            p_id = c_id = self.root_cid[lid]
            res = new_loc.select(id=lid)
            for k in ['locator1', 'locator2', 'locator3']:
                if (r[k] == NULL or r[k] == ''):
                    continue
                type_id = res and res[1]['%s_type_id' % k]
                type_name = c_types.select(id=type_id)[1]['name']
                c_name = '%s %s' % (type_name, r[k])
                key = (lid, p_id, type_id, c_name)
                c_id = ctid.get(key)
                if c_id is None:
                    # We use +2 here because container
                    # '1' is the root container, needed
                    # for referential integrity.
                    c_id = containers.append([p_id, type_id, c_name]) + 2
                    container_location.append([lid, c_id])
                    ctid[key] = c_id
                p_id = c_id
            alid[r['aliquot_id']] = c_id

        containers['commit_message'] = COMMIT_MESSAGE
        # Root container, for integrity
        containers.insert(0, [NULL, NULL,
                              'Root Container', 'Root Container'])
        containers['id'] = '%(id)s'
        for k in ['locator1_desc', 'locator2_desc', 'locator3_desc']:
            del new_loc[k]
        for k in ['locator1_type_id', 'locator2_type_id', 'locator3_type_id']:
            del new_loc[k]
        del new_loc['root_type_id']

def main():
    tables = csv_read_dir(_prefix)
    order = tables.keys()
    del order[order.index('aliquot')]
    del order[order.index('location')]
    del order[order.index('valid_technicians')]
    del order[order.index('sample')]
    order.append('valid_technicians')
    order.append('sample')
    order.append('location')
    order.append('aliquot')
    writer = Writer(tables, order)
    writer.crunch()

if __name__ == '__main__':
    main()
