# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: container.py 749 2005-06-28 22:21:49Z sidnei $
"""

__metaclass__ = type

from zope.app import zapi
from zope.app.rdb.interfaces import DatabaseException

from sip.browser import api
from sip.csv.lookup import getContainerId
from sip.csv.processor.common import Processor
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent, ObjectCreatedEvent


class ModifyContainer(Processor):
    """Process ``modify_containers.csv`` doing the necessary
    modifications to existing containers"""

    def process(self, context, data, request):
        stats ={'containers':0,
                'errors':0}
        container_factory = api.factory('sip.Container', context=context)
        project = api.getProject(context)
        for move in data.get('reparent_container', ()):
            if not move.get('destination_path'):
                continue
            container = api.contained(
                container_factory.get(move['container_id']),
                project, 'Container.%s' % move['container_id'])
            new_parent = container_factory.get(
                getContainerId(context, move['destination_path']))
            if new_parent.id == container.parent_id:
                print >> self.log, ('Not reparenting container %s to %s '
                                    ' - it was already there' %
                                    (container.name, new_parent.name))
                continue
            try:
                container.move_to(new_parent)
            except Exception, exc:
                stats['errors'] += 1
                print >> self.log, ('An error occurred while trying '
                                    'to reparent container %s to %s.' %
                                    (container.name, new_parent.name))
                if isinstance(exc, DatabaseException):
                    raise
                import traceback
                traceback.print_exc(file=self.log)
            else:
                stats['containers'] += 1
                print >> self.log, ('Reparent container: %s (to %s). ' %
                                    (container.name, new_parent.name))
        return stats

class AddContainer(Processor):
    """Process ``add_containers.csv`` adding the required containers"""

    def process(self, context, data, request):
        stats ={'containers':0,
                'errors':0}
        message = request.form.get('commit_message')
        if not message:
            if request.principal:
                message = "Added by '%s'" % request.principal.id
            else:
                message = "(Empty commit message)"
        d_containers = data.get('container', None)
        container_factory = api.factory('sip.Container', context=context)
        containertype_factory = api.factory('sip.ContainerType', context=context)
        for container in d_containers:
            del container['project_name']
            container['commit_message'] = message
            type_id = container['type_id']
            if not type_id:
                stats['errors'] += 1
                print >> self.log, ('An error occurred while trying to add a '
                                    'new container. The container_type is '
                                    'missing.')
                continue
            name = container['container_name']
            name = name.strip()
            del container['container_name']
            rows = container['rows']
            del container['rows']
            cols = container['columns']
            del container['columns']

            if not name:
                name = containertype_factory.select('container_type_id=%d'
                                                    % type_id)[0].name
            name = name.strip()
            try:
                if rows:
                    rows = int(rows)
                else:
                    rows = 1
                if cols:
                    cols = int(cols)
                else:
                    cols = 1
            except ValueError:
                stats['errors'] += 1
                print >> self.log, ('An error occured while trying to add a '
                                    'new container. Rows and columns must be '
                                    'integers.')
                continue
            path = container['container_path']
            del container['container_path']
            location_name = path[len('path://'):]
            if location_name:
                location_name = location_name.split('/')[0]
            parent_id = getContainerId(context, path)
            container['parent_id'] = parent_id
            for r in range(1,rows+1):
                for c in range(1,cols+1):
                    if rows == 1 and cols == 1:
                        container['name'] = name
                    else:
                        container['name'] = '%s-%dx%d' % (name, r, c)
                    try:
                        new_container = container_factory(**container)
                        notify(ObjectCreatedEvent(new_container))
                    except Exception, exc:
                        stats['errors'] += 1
                        print >> self.log, ('An error occurred while trying '
                                            'to add a new container.')
                        if isinstance(exc, DatabaseException):
                            raise
                    else:
                        stats['containers'] += 1
                        print >> self.log, ('Add new container: %s (%s). ' %
                                            (new_container.id,
                                             new_container.name))
        return stats
