# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: subscriber.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.app.tests import setup
from zope.app import zapi
from zope.app.servicenames import Utilities, \
     Authentication
from zope.app.utility import LocalUtilityService
from zope.app.pluggableauth import PluggableAuthenticationService

from sip.interfaces import IAppRoot
from sip.interfaces.utilities import ISplitDerive, IQuery
from sip.utilities.splitderive import SplitDerive
from sip.utilities.query import Query
from sip import SecurityPrincipal, SecurityRole

from sqlos.auth import SQLObjectPrincipalSource
from sqlos.interfaces import ISQLObject

def appRootCreationSubscriber(event):
    obj = event.object
    if not IAppRoot.providedBy(obj): return

    # Create a service manager at the approot level
    mgr = setup.createServiceManager(obj)

    # Create a pluggable auth service at the root folder
    setup.addService(mgr, Authentication,
                     PluggableAuthenticationService())

    # Then add a SQLObject PrincipalSource
    auth = zapi.getService(Authentication, context=obj)
    prin_source = SQLObjectPrincipalSource()
    auth.addPrincipalSource('sqlob', prin_source)

    # Setup Utility and Interface Services
    setup.addService(mgr, Utilities, LocalUtilityService())

    # Setup our own utilities
    setup.addUtility(mgr, '', ISplitDerive, SplitDerive(), 'split-derive')
    setup.addUtility(mgr, '', IQuery, Query(), 'query')

def sqlobjectModifiedSubscriber(event):
    obj = event.object
    if not ISQLObject.providedBy(obj): return

    if isinstance(obj, (SecurityPrincipal, SecurityRole)):
        from sip.security.source import securityCache
        securityCache.clear()
        from sip.security.principalrole import securityCache
        securityCache.clear()
