# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: fields.py 759 2005-06-28 23:45:42Z sidnei $
"""

__metaclass__ = type

import sys
from zope.interface import implements
from zope.app import zapi
from zope.schema import Text, Field, Int, TextLine, List, Choice
from sip.interfaces.fields import ICommitMessage, IBRField, \
    ISubjectTypeField, IInAndOutField
from sqlos.utils import fromItems

class NullAddFieldAdapter:

    def __init__(self, context):
        self.context = context

    def __getattr__(self, attr):
        if attr == 'context':
            return self.context
        return getattr(self.context, attr)

    def __setattr__(self, attr, value):
        if attr == 'context':
            return object.__setattr__(self, attr, value)
        return object.__setattr__(self.context, attr, value)

class InAndOutField(List):
    """marker field"""
    implements(IInAndOutField)

class CommitMessage(Text):
    """ Just a 'marker' field for using a separate
    widget that is always empty by default """

    implements(ICommitMessage)

    def __init__(self, *args, **kw):
        super(CommitMessage, self).__init__(*args, **kw)
        self.order = sys.maxint


class SubjectTypeField(Choice):
    implements(ISubjectTypeField)


class BRField(Field):
    implements(IBRField)


def FieldFactory(klass, *args, **kw):
    constraint = kw.get('constraint', None)
    if kw['constraint']: del kw['constraint']
    return type('Constrained%s' % klass.__name__, (klass,),
                {'constraint':constraint})(*args, **kw)


class ContainerTypeField(Choice):

    def get(self, object):
        from sip.interfaces.location import ILocation
        if ILocation.providedBy(object):
            object = object.getRootContainer()
        return getattr(object, self.__name__)

    def query(self, object, default=None):
        from sip.interfaces.location import ILocation
        if ILocation.providedBy(object):
            object = object.getRootContainer()
        return getattr(object, self.__name__, default)

    def set(self, object, value):
        from sip.interfaces.location import ILocation
        if self.readonly:
            raise TypeError("Can't set values on read-only fields "
                            "(name=%s, class=%s.%s)"
                            % (self.__name__,
                               object.__class__.__module__,
                               object.__class__.__name__))
        if ILocation.providedBy(object):
            object = object.getRootContainer()
        setattr(object, self.__name__, value)

class FieldTypeField(Choice):

    def __init__(self, **kw):
        if kw.has_key('allowed_names'):
            self._names = kw.get('allowed_names', None)
            del kw['allowed_names']
        super(FieldTypeField, self).__init__(**kw)

    def filterfields(self, item):
        if not self._names:
            return True
        return item[0] in self._names

    def bind(self, object):
        clone = super(FieldTypeField, self).bind(object)
        clone.vocabulary = clone.vocabulary(object, self)
        return clone

# We monkeypatch `default` for the field interfaces we use just so we
# don't need to duplicate lots of code.
from zope.schema.interfaces import IText, ITextLine, IBool, \
     IInt, IFloat, IDate, IDatetime

def monkeypatch():
    for iface in (IText, ITextLine, IBool, IInt, IFloat,
                  IDate, IDatetime):
        iface['default'].required = False

# Run the monkeypatch
monkeypatch()
