# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: test_csvviews.py 749 2005-06-28 22:21:49Z sidnei $
"""

import unittest
from transaction import get_transaction
from zope.app import zapi
from zope.app.container.interfaces import IReadContainer
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent
from sip.ftests.base import Base
from os import curdir
from os.path import join, abspath, dirname, split, basename, splitext
from sip.browser import api

try:
    __file__
except NameError:
    # Called directly, so no __file__ global exists.
    _prefix = abspath(curdir)
else:
    # Called by another script.
    _prefix = abspath(dirname(__file__))


class TestCSVViews(Base):
    level = 2
    use_test_user = False

    def _setup(self):
        self.project = project = self._addProject()
        ct = self.root_ct = self._addContainerType()
        location = self.location = self._addLocation(ct.id)
        self.root_container = location.getRootContainer()
        ct = self.ct = self._addContainerType(root=False, name='box')
        self.container = self._addContainer(location,ct.id)
        project.valid_locations = [location.id]
        stype = self.stype = self._addSampleType(project)
        tech = self.tech = self._addTechnician()
        sample = self.sample = self._addSample(project, stype.id, tech.id)
        aliquot = self.aliquot = self._addAliquot(project, sample.id)
        aliquot2 = self.aliquot2 = self._addAliquot(project, sample.id)

    def _teardown(self):
        attrs = ['aliquot', 'aliquot2', 'container', 'ct', 'location',
                'root_container', 'root_ct', 'sample', 'tech', 'stype',
                'project',
                ]
        get_transaction().abort()
        factory = api.factory('sip.ContainerLocation', context=self.sip)
        res = factory.select()
        for obj in res:
            notify(ObjectModifiedEvent(obj))
            obj.destroySelf()
            del obj
        project = IReadContainer(self.project)
        for obj in project['locationsproject'].values():
            notify(ObjectModifiedEvent(obj))
            obj.destroySelf()
            del obj
        for obj in project['schemas'].values():
            notify(ObjectModifiedEvent(obj))
            obj.destroySelf()
            del obj
        del project
        factory = api.factory('sip.SampleTypeProject', context=self.sip)
        res = factory.select()
        res = [r for r in res]
        for obj in res:
            notify(ObjectModifiedEvent(obj))
            obj.destroySelf()
            del obj
        for attr in attrs:
            obj = getattr(self, attr, None)
            if obj is not None:
                delattr(self,attr)
                notify(ObjectModifiedEvent(obj))
                obj.destroySelf()
                del obj
        get_transaction().commit()


    def testGenerateContainerReparentCSV(self):
        self._setup()
        project = self.project
        request = self.request
        request.form['container_ids'] = [self.container.id]
        reparent_csv = zapi.getView(project,'modify_containers.csv', request)
        self.failUnless(reparent_csv())

    def testDeriveCSVGeneration(self):
        self._setup()
        project = self.project
        tech = self.tech
        alids = [self.aliquot.id, self.aliquot2.id]
        samples = zapi.traverse(project, 'samples')
        adding = zapi.getView(samples, '+', self.request)
        self.request.form['aliquot_ids'] = alids
        csv = zapi.getView(project, 'derive_sample.csv', self.request)
        self.failUnless(csv())

    def testSplitCSVGeneration(self):
        self._setup()
        request = self.request
        project = self.project
        id = self.aliquot.id
        request.form['aliquots'] = [{'id': id,
                                     'split_count': '2'}]
        csv = zapi.getView(project, 'split_aliquot.csv', request)
        self.failUnless(csv())

    def testCSVFileUpload(self):
        # can't do it because the first time we try import around we
        # clear the objects we set up
        self._setup()
        get_transaction().commit()
        request = self.request
        project = self.project
        filename = join(_prefix, 'data', 'add_containers.csv')
        request.form['importfile'] = open(filename,'r')
        view = zapi.getView(project,'confirm_import.html',request)
        location = IReadContainer(self.location)
        self.failUnless(len(location)==1)
        view.tryImport()
        del request.form['importfile']
        request.form['template'] = 'add_containers.csv'
        request.form['tid'] = view.tid
        view.finishImport()
        self.failUnless(len(location)==2)
        self._teardown()


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestCSVViews))
    return suite

if __name__ == '__main__':
    unittest.main()
