# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/


import unittest

from transaction import get_transaction
from zope.schema import getFieldsInOrder, getFieldNamesInOrder
from zope.app import zapi

from sip.schema.tests.test_schemautility import SchemaUtilityTests
from sip.schema.interfaces import IMutableSchema

from sip.ftests.base import Base
from sip.browser import api

class SQLSchemaUtilityTest(SchemaUtilityTests, Base):
    level = 2
    use_test_user = False

    def setUp(self):
        Base.setUp(self)
        try:
            SchemaUtilityTests._additionalSetup(self)
        except:
            get_transaction().abort()
            raise

    def _createSchemaUtility(self):
        pf = api.factory('sip.Project', context=self.sip)
        p = pf(name='SchemaTestProject', commit_message='Test')
        stf = api.factory('sip.SampleType', context=self.sip)
        st = stf(name='Blood', commit_message='Test')
        factory = api.factory('sip.SampleTypeSchema', context=self.sip)
        schema = factory(sample_type_id=st.id,
                             project_id=p.id)

        sts = api.contained(schema, self.sip, 'schema')
        return IMutableSchema(sts)

    def test_traverseToField(self):
        s = self.s
        s.addField(u'alpha', self.alpha)
        f1 = s[u'alpha']
        order = f1.order
        f1 = zapi.traverse(s, 'alpha')
        self.assertEquals(f1.order, self.alpha.order)
        title = zapi.traverse(f1, 'title')
        self.assertEquals(title, self.alpha.title)
        fields = getFieldsInOrder(s)
        for k, v in fields:
            self.failUnless(v.title is not None)

    def test_schemaFieldItems(self):
        s = self.s
        s.addField(u'alpha', self.alpha)
        fields = getFieldNamesInOrder(s)
        for k, v in s._attrs.items():
            self.failUnless(k in fields, (k, fields))

    def test_default_not_required(self):
        from zope.schema.interfaces import IText, ITextLine, IBool, \
             IInt, IFloat, IDate, IDatetime
        for iface in (IText, ITextLine, IBool, IInt, IFloat,
                      IDate, IDatetime):
            self.failUnless(iface['default'].required == False)

    def tearDown(self):
        Base.tearDown(self)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(SQLSchemaUtilityTest))
    return suite

if __name__ == '__main__':
    unittest.main()
