# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: __init__.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.interface import Interface, Attribute

from zope.schema import TextLine, Text, Datetime, Int
from zope.schema import Choice, Bool, Password

from zope.app import zapi
from zope.app.container.interfaces import IReadContainer
from zope.app.site.interfaces import IPossibleSite

from sip.i18n import _
from sip.utils import Unique, PasswordMatch
from sip.descriptors import BoundCallableDescriptor
from sip.fields import CommitMessage, BRField, SubjectTypeField, FieldFactory

from sqlos.interfaces import ISQLAttributeAnnotatable
from sqlos.interfaces.container \
     import ISQLObjectContainer

class INoopBreadcrumb(Interface): pass

class INolinkBreadcrumb(Interface): pass

class IAppRoot(IReadContainer, IPossibleSite):
    """App Root is a container with 'virtual' subcontainers, registered
    with the AppContainerRegistry service"""

class IAppProjects(Interface):
    """Available projects on the App"""

    available_projects = Choice(
        title=_('Available Projects'),
        required=True,
        vocabulary='sip.vocab.available_projects')

class IAppPrincipals(Interface):
    """Available principals on the App"""

    available_principals = Attribute('Available Principals')

class IAppLocations(Interface):
    """Available locations on the App"""

    available_locations = Choice(
        title=_('Available Locations'),
        required=True,
        vocabulary='sip.vocab.locations')

class IAppContainerTypes(Interface):
    """Available Container Types on the App"""

    available_containertypes = Choice(
        title=_('Available Container Types'),
        required=True,
        vocabulary='sip.vocab.available_container_types')

class IAvailableSampleTypes(Interface):
    """ Sample Types Available for a Project """

    available_sampletypes = Choice(
        title=_('Available Sample Types'),
        vocabulary='sip.vocab.sample_types')

class ISQLSchema(ISQLAttributeAnnotatable):
    """ SQLObject-based schemas must declare 'id' or else
    it doesn't get security set """

    id = Attribute('Id')

class ISIPSchema(ISQLSchema):

    commit_message = CommitMessage(title=_('Edit Comment'),
                                   description=_('A message explaining '
                                                 'the changes you made to '
                                                 'this object'),
                                   required=True)

class IIterator(Interface):

    def __iter__():
        ''' '''

    def next():
        ''' '''
