# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: add.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.interface import Interface
from zope.schema import TextLine, Bool, Text, Field

from sip.i18n import _
from sip.fields import FieldTypeField

class IFieldAdd(Interface):

    id = TextLine(
        title=_('Name'),
        default=u"",
        required=True,
        )

    type = FieldTypeField(
        title=_('Field Type'),
        required=True,
        allowed_names=['Text Field',
                       'Integer Field',
                       'Boolean Field',
                       'Date Field',
                       'Text Line Field',
                       'Float Field',
                       ],
        vocabulary='sip.vocab.field_types'
        )

    title = TextLine(
        title=_('Title'),
        description=_('A short summary or label'),
        default=u"",
        required=True,
        )

    description = Text(
        title=_('Description'),
        description=_('A description of the field'),
        default=u"",
        required=False,
        )

    required = Bool(
        title=_('Required'),
        description=_('Whether a field requires its value to exist.'),
        default=True,
        required=False)

    usetales = Bool(
        title=_('Use TALES Validation?'),
        description=_('Should this field allow TALES Validation?'),
        required=False,
        default=False
        )

    default = TextLine(
        title=_('Default Value'),
        description=_('Default value for this field, if none is provided'),
        required=False,
        default=u''
        )
