# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: csv.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.interface import Interface

class IProcessor(Interface):
    """Receives some input, and takes actions based on it.

    There are two modes of operation:

    - Summary Mode -- Report actions to be taken, but don't
      make any change to the real data
    - Real Mode -- Perform all actions and save, removing
      temporarily stored data from a ``Summary Mode`` run
      if any.
    """

    def __init__(template):
        """Nothing interesting here """

    def __call__(context=None, dry_run=True, data=None, tid=None):
        """Runs the processor, storing the summary for output
        if in dry_run mode, or just exceptions and warnings if
        not dry run.

        In both cases, a string containing the information of the
        performed actions is returned.

        If ``data`` is not passed in, we try to load data using
        the ``tid`` passed in.
        """

    def process():
        """Does the actual processing. Called by ``__call__``."""

class IMassager(Interface):
    """Receives some data and does some pre-processing
    to make the job easier for the Processor"""

    def __init__(filename):
        """Nothing interesting here"""

    def __call__(context, data, **kw):
        """Do the massaging and return processed data"""


class ITempStorage(Interface):
    """Used for storing temporary info for a process run, using a
    ticket id as the key
    """

    def store(context, data, proc_id):
        """Stores ``data``, which may be a file-like object and ``proc_id``,
        which is the identifier of the processor used to handle ``data``.

        Return a ticket id to be used later for retrieving the data back.
        """

    def load(context, ticket_id):
        """Returns a tuple with (data, proc_id), given a ticket_id.

        If the ticket id is invalid or expired, a KeyError is raised.
        """

    def expire(context, ticket_id):
        """Expires the data for a given ticket id, invalidating the
        entries and removing any stored data.
        """

class IFileStorage(ITempStorage):
    """Stores data on a temporary file"""

class ISQLStorage(ITempStorage):
    """Stores data on a temporary table in SQL"""

class ISessionStorage(ITempStorage):
    """Stores data on a session variable"""

class ILookupHelper(Interface):
    """A marker interface for value lookup utilities.

    This is used by the massager, together with a field
    name, a template name and section name to translate
    the value of a csv to something that can be used to insert
    new records."""

    def newName(self):
        """Returns the new column name"""

    def lookup(self, context, value):
        """Looks up the value to be used for
        a field, given the original value and a context"""
