# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: location.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.interface import Interface
from zope.schema import TextLine, Int, Bool
from sip.fields import ContainerTypeField
from sip.interfaces import ISIPSchema, Unique, FieldFactory
from sip.descriptors import BoundCallableDescriptor
from sip.i18n import _

class ILocation(Interface):
    """A marker interface for locations"""

    def getRootContainer():
        """ Returns the first container in this location which has
        no parent (thus, being a root container)"""

    def getRootId():
        """ returns the container_id of the root container"""

class ILocationSchema(ISIPSchema):
    """ """
    name = FieldFactory(
        TextLine,
        title=_('Location Name'),
        description=_('A string to serve as identifier for this location'),
        required=True,
        max_length=50,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.Location', columnName='name'))
        )

    type_id = ContainerTypeField(
        title=_('Container Type'),
        description=_('The container type for this location'),
        vocabulary='sip.vocab.container_types')

    building = TextLine(title=_('Building'),
                        description=_('Building where this location pertains'),
                        required=True,
                        max_length=255,
                        )
    room = TextLine(title=_('Room'),
        description=_('Room where this location is located inside the building'),
                        required=True,
                        max_length=255,
                        )
    temperature = Int(title=_('Temperature'),
                           description=_('The temperature for this location'),
                      required=False)
