# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: principal.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.interface import Interface
from zope.schema import TextLine, Password
from sip.i18n import _
from sqlos.interfaces.auth import IPrincipal as IPrincipalBase
from sip.interfaces import ISQLSchema, FieldFactory
from sip.descriptors import BoundCallableDescriptor
from sip.utils import Unique, PasswordMatch

class IPrincipalInfo(ISQLSchema):
    """ A Principal! """

    fullname = TextLine(title=_('Full Name'),
                        required=True)

    username = FieldFactory(
        TextLine,
        title=_('Username'),
        description=_('The user login name'),
        max_length=50,
        required=True,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.Principal', columnName='username'))
        )

class IPrincipalPassword(Interface):
    """ Split into a different interface to ease protecting
    the password by a different permission """

    password = Password(title=_('Password'),
                        required=True)

    confirm_pass = FieldFactory(
        Password,
        title=_('Confirm Password'),
        required=True,
        constraint=BoundCallableDescriptor(
        PasswordMatch())
        )

class IPrincipal(IPrincipalInfo, IPrincipalPassword, IPrincipalBase):
    """ Principal Schema """
