# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: sample.py 749 2005-06-28 22:21:49Z sidnei $
"""

from zope.interface import Interface, Attribute
from zope.schema import TextLine, Text, Datetime, Bool
from zope.schema import Choice
from sip.fields import SubjectTypeField
from sip.i18n import _
from sip.interfaces import ISIPSchema, Unique, FieldFactory
from sip.descriptors import BoundCallableDescriptor

from calendar_widget import CalendarDate

class ISampleSchema(ISIPSchema):
    """ """

    name = FieldFactory(
        TextLine,
        title=_('Sample Name'),
        description=_("The name of the sample"),
        max_length=255,
        required=False,
        constraint=BoundCallableDescriptor(
        Unique(className='sip.Sample', columnName='name', byProject=True))
        )

    other_id = TextLine(
        title=_('Other ID'),
        description=_("A outside parties identification for this sample"),
        max_length=255,
        required=False)

    type_id = Choice(
        title=_('Sample Type'),
        description=_('The type (kind) of this sample'),
        required=True,
        vocabulary=u'sip.vocab.project_sample_types')

    technician_id = Choice(
        title=_('Technician ID'),
        description=_('The technician who drew this sample.'),
        readonly=False,
        required=False,
        vocabulary='sip.vocab.project_technicians')

    subject_type_id = SubjectTypeField(
        title=_('Subject Type'),
        description=_('Type of subject'),
        readonly=False,
        required=True,
        default=1,
        vocabulary='sip.vocab.subject_types')

    subject_id = TextLine(
        title=_('Subject ID'),
        description=_("Unique identifier for this subject"),
        required=False)

    collection_date = CalendarDate(
        title=_('Date Collected'),
        description=_('Date this sample was collected'),
        required=False)

    comments = Text(
        title=_('Comments'),
        description=_("Ad hoc comments about this sample."),
        required=False)

class ISampleDate(Interface):
    """ """

    date_generated = Datetime(
        title=_('Date Generated'),
        description=_('Date this sample was generated'),
        required=False)

    derived = Attribute('Sample is Derived')


class ISample(ISampleSchema, ISampleDate):
    """ """

    derived = Attribute('Sample is Derived')

class ISampleAdd(Interface):
    """ Marker interface for adding a sample """
