# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/



__metaclass__ = type

from sqlos import SQLOS
from zope.interface import implements
from sqlobject import *
from sip.interfaces.shippinginfo import IContactInfo, IShippingInfo, ICarrier,\
        IFundingSource, IFundingInfo, ITempMethod

class ContactInfo(SQLOS):

    implements(IContactInfo)

    _idName = 'contact_info_id'

    _columns = [
            StringCol('firstname', length=255, notNull=0),
            StringCol('lastname', length=255, notNull=0),
            StringCol('institution', length=255, notNull=0),
            StringCol('address1', length=255, notNull=0),
            StringCol('address2', length=255, notNull=0),
            StringCol('city', length=255, notNull=0),
            StringCol('state', length=255, notNull=0),
            StringCol('zipcode', length=255, notNull=0),
            StringCol('country', length=255, notNull=0),
            StringCol('phone', length=255, notNull=0),
            StringCol('fax', length=255, notNull=0),
            StringCol('email', length=255, notNull=0),
            StringCol('project', length=255, notNull=0),
            StringCol('objective', notNull=0),
            StringCol('commit_message', notNull=1),
            ]


class ShippingInfo(SQLOS):

    implements(IShippingInfo)

    _idName = 'shipping_info_id'

    _columns = [
            BoolCol('hand_delivered', notNull=0),
            ForeignKey('Carrier', name='carrier_id', cascade=False),
            StringCol('tracking_number', length=255, notNull=0),
            StringCol('account_number', length=255, notNull=0),
            ForeignKey('TempMethod', name='temp_method_id', cascade=False),
            StringCol('housenumber', length=255, notNull=0),
            ForeignKey('Technician', name='technician_id', cascade=False),
            BoolCol('newcontainer', notNull=0, default=False),
            StringCol('instructions', length=255, notNull=0),
            StringCol('commit_message', notNull=1),
            ]

class Carrier(SQLOS):

    implements(ICarrier)

    _idName = 'carrier_id'

    _columns = [
        StringCol('name', length=255, notNull=1),
        ]

class FundingSource(SQLOS):

    implements(IFundingSource)

    _idName = 'funding_source_id'

    _columns = [
            StringCol('name', length=255, notNull=1),
            BoolCol('federal', notNull=1, default=False),
            ]

class FundingInfo(SQLOS):

    implements(IFundingInfo)

    _idName = 'funding_info_id'

    _columns = [
            ForeignKey('Shipment', name='shipment_id', cascade=False),
            ForeignKey('FundingSource', name='funding_source_id',
                    cascade=False),
            StringCol('grant_number', length=255, notNull=0),
            DecimalCol('cost', size=14, precision=2, notNull=0),
            StringCol('agency', length=255, notNull=0),
            StringCol('principal', length=255, notNull=0),
            StringCol('state', length=255, notNull=0),
            DateCol('enddate', notNull=0),
            DecimalCol('fee', size=9, precision=2, notNull=0),
            StringCol('commit_message', notNull=1),
            ]

class TempMethod(SQLOS):

    implements(ITempMethod)

    _idName = 'temp_method_id'

    _columns = [
            StringCol('name', length=255, notNull=1),
            ]
