# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: __init__.py 749 2005-06-28 22:21:49Z sidnei $
"""

from itertools import imap, chain, ifilter

from zope.interface import implements
from zope.proxy import removeAllProxies
from zope.app import zapi
from zope.app.container.contained import Contained
from zope.app.container.constraints import checkFactory
from zope.component.exceptions import ComponentLookupError
from zope.component import IFactory

from sip.browser import api
from sip.interfaces import INoopBreadcrumb
from sip.interfaces.source import ISource

from sqlos.interfaces import ISQLObject, ISQLAttributeAnnotatable
from sqlos.interfaces.container import ISQLObjectContainer
from sqlobject.main import SQLObjectNotFound

_instance_cache = {}

class Singleton(object):

    def __new__(cls, *args, **kw):
        cache = _instance_cache
        key = (cls, args, tuple(kw.iteritems()))
        if key in cache:
            return cache[key]
        obj = object.__new__(cls)
        obj.__init__(*args, **kw)
        cache[key] = obj
        return obj

class Source(Contained, Singleton):

    implements(ISQLObjectContainer, ISource,
               ISQLAttributeAnnotatable, INoopBreadcrumb)

    use_project = True

    def __init__(self, use_project=None,
                 constraint=None, context=None):
        if context is not None:
            self.__parent__ = context
        if use_project is not None:
            self.use_project = use_project
        if constraint is not None:
            self.constraint = constraint

    def allowedFactories(self):
        items = zapi.getFactoriesFor(ISQLObject, context=self)
        names = [name for name, factory in items]
        f = lambda name: (name, zapi.getUtility(IFactory, name))
        factories = map(f, names)
        a = lambda f, container=self: (checkFactory(container, None, f[1])
                                       and f[0] or None)
        allowed = filter(None, map(a, factories))
        return allowed

    def getQuery(self, className):
        query = None

        parent = api.queryRoot(self)

        if parent is None:
            return '1 = 1'

        factory = api.factory(className, context=self)
        p_key = getattr(factory, '_idName', 'id')

        if self.use_project:
            parent = api.queryProject(self)

            if parent is not None:
                projectid = parent.id
                query = 'project_id = %s' % projectid
        additional = getattr(self, 'query', None)
        if query is not None:
            if additional:
                query = query + ' ' + additional
            return query

        return '1 = 1'

    def keys(self):
        return [k for k, v in self.items()]

    def __iter__(self):
        return iter(self.keys())

    def values(self):
        return [v for k, v in self.items()]

    def items(self):
        result = []
        for f_name in self.allowedFactories():
            items = []
            query = self.getQuery(f_name)
            factory = api.factory(f_name, context=self)
            c = lambda p: ("%s.%s" % (f_name, p.id), p)
            items = imap(c, factory.select(clause=query))
            items = api.containedItems(items, self)
            constraint = getattr(self, 'constraint', None)
            if constraint:
                items = ifilter(constraint, items)
            items = api.itemsCanView(items, self, factory)
            result.append(items)

        return chain(*result)

    def __getitem__(self, name):
        try:
            parts = name.split('.')
            id = int(parts[-1])
            factoryName = '.'.join(parts[:-1])
        except ValueError:
            raise KeyError, name

        if factoryName not in self.allowedFactories():
            raise KeyError, name

        factory = api.factory(factoryName, context=self)
        try:
            item = factory.get(id)
        except SQLObjectNotFound:
            raise KeyError, name

        if self.use_project:
            parent = api.queryProject(self)
            if parent is None:
                raise KeyError, name
            if not item.project_id == parent.id:
                raise KeyError, name

        item = api.contained(item, self, name=name)
        items = [("%s.%s" % (factoryName, item.id), item)]

        constraint = getattr(self, 'constraint', None)
        if constraint:
            items = filter(constraint, items)
        items = [(k, v) for k, v in items if api.canView(v)]

        if len(items):
            return items[0][1]
        raise KeyError, name

    def get(self, name, default=None):
        try:
            item = self[name]
            return item
        except KeyError:
            return default

    def __contains__(self, name):
        return self.get(name, None) is not None

    def __len__(self):
        return len(self.keys())

    def __delitem__(self, name):
        raise KeyError, "It's not possible to delete items. %s" % name

    def __setitem__(self, name, content):
        return name

def createSource(context, use_project=True, constraint=None):
    return Source(use_project=use_project,
                  constraint=constraint,
                  context=context)
