
CREATE OR REPLACE FUNCTION
clear_project(VARCHAR) RETURNS INTEGER AS '
 DECLARE
    -- aliases
    pname ALIAS FOR $1;

    -- variables
    projectid INTEGER;
    sampleid INTEGER;
 BEGIN
 
    SELECT INTO projectid project_id from project
    WHERE name=pname;

    DELETE FROM origin_sample_aliquots
    WHERE aliquot_id IN (select aliquot_id FROM aliquot
                         WHERE project_id=projectid);

    DELETE FROM aliquot
    WHERE project_id=projectid;
  
     DELETE FROM additional_sample_data
     WHERE sample_id IN (SELECT sample_id FROM sample
                         WHERE project_id=projectid);

    DELETE FROM sample WHERE project_id=projectid;

    RETURN 0;
 END
' LANGUAGE 'plpgsql';

CREATE OR REPLACE FUNCTION
get_field_id(INTEGER,VARCHAR,VARCHAR) RETURNS INTEGER AS '
 DECLARE
    -- aliases
    sampleid ALIAS FOR $1;
    fname ALIAS FOR $2;
    stname ALIAS FOR $3;

    -- variables
    field_id INTEGER;
    projid INTEGER;
    stypeid INTEGER;
    schemaid INTEGER;

 BEGIN
    SELECT INTO projid, stypeid project_id, type_id
    FROM sample
    WHERE sample_id=sampleid;

    IF stname != '''' THEN
       SELECT INTO stypeid sample_type_id FROM sample_type
       WHERE name = stname;
    END IF;

    SELECT INTO schemaid sample_type_schema_id FROM sample_type_schema
    WHERE project_id=projid
        AND sample_type_id=stypeid;

    SELECT INTO field_id schema_field_id FROM schema_field
    WHERE name=fname
        AND schema_id=schemaid;

    RETURN field_id;
 END;

' LANGUAGE 'plpgsql';

CREATE OR REPLACE FUNCTION
select_on_info(INTEGER,VARCHAR,TEXT) RETURNS SETOF INTEGER AS '
 DECLARE
    -- aliases
    field_id ALIAS FOR $1;
    op ALIAS FOR $2;
    value ALIAS FOR $3;

    -- constants
    int_type CONSTANT VARCHAR := ''zope.schema._bootstrapfields.Int'';
    float_type CONSTANT VARCHAR := ''zope.schema._field.Float'';
    date_type CONSTANT VARCHAR := ''zope.schema._field.Date'';
    bool_type CONSTANT VARCHAR := ''zope.schema._bootstrapfields.Bool'';
    text_type CONSTANT VARCHAR := ''zope.schema._bootstrapfields.Text'';
    textline_type CONSTANT VARCHAR := ''zope.schema._bootstrapfields.TextLine'';

    -- variables
    data_type VARCHAR;
    convert_type VARCHAR;
    query VARCHAR;
    query_rec RECORD;

 BEGIN
    SELECT INTO data_type type FROM schema_field
    WHERE schema_field_id = field_id;
    IF data_type = int_type THEN
       convert_type := ''INTEGER'';
    ELSIF data_type = float_type THEN
       convert_type := ''NUMERIC'';
    ELSIF data_type = date_type THEN
       convert_type := ''DATE'';
--    ELSIF data_type = bool_type THEN
--       convert_type := ''BOOLEAN'';
    ELSE
       convert_type := ''VARCHAR'';
    END IF;
    CREATE TEMP TABLE tmp_additional_sample_data
    AS SELECT * FROM additional_sample_data
       WHERE schema_field_id=field_id;
    query = ''SELECT sample_id FROM tmp_additional_sample_data WHERE '' || ''CAST(value AS '' || convert_type || '') '' || op || ''CAST('' || quote_literal(value) || '' AS '' || convert_type || '') '';
    -- RAISE NOTICE ''query = %'',query;
    FOR query_rec IN EXECUTE query LOOP
       RETURN NEXT query_rec.sample_id;
    END LOOP;
 DROP TABLE tmp_additional_sample_data;
 RETURN;
 END;
 ' LANGUAGE 'plpgsql';


-- get_aliquot_ids
-- INPUT: an integer representing the number of aliquot_id's to return
-- OUTPUT: a string containing the aliquot_id's seperated by spaces
-- SIDE EFFECTS: entries in the aliquot_alocation table that are older than
--     hold interval are freed up. aliquot_id's that are returned have there
--     entries in the alocation table set to reserved and their timestamps
--     set to now
--
-- must be used in conjunction with the trigger function remove_id()


CREATE OR REPLACE FUNCTION get_aliquot_ids(INTEGER) RETURNS SETOF INTEGER AS '
 DECLARE

  count ALIAS FOR $1; -- number of ids to return

  free CONSTANT TEXT := ''free'';
  reserved CONSTANT TEXT := ''reserved'';
  hold_interval CONSTANT INTERVAL := ''10 minutes'';

  free_id RECORD;
  id INTEGER;
  tmp_count INTEGER;

 BEGIN
   tmp_count := count;
   -- free up any reserved ids that are older than the hold_interval
   UPDATE aliquot_allocation
    SET status = free
    WHERE status = reserved
     AND age(now(),tstamp) >= hold_interval;

   -- get any free ids and set their status to reserved and the timestamp to
   -- now()
   FOR free_id IN SELECT * FROM aliquot_allocation WHERE status  = free LOOP
     UPDATE aliquot_allocation
      SET status = reserved,
          tstamp = now()
      WHERE aliquot_id = free_id.aliquot_id;
     RETURN NEXT free_id.aliquot_id;
     tmp_count := tmp_count - 1;
     EXIT WHEN tmp_count = 0;
   END LOOP;

   -- if we found enough ids return them
   IF tmp_count = 0 THEN
     RETURN;
   END IF;

   -- if we still need more ids get them from the sequence
   FOR tmp IN 1..tmp_count LOOP
     id := nextval(''public.aliquot_aliquot_id_seq''::text);
     INSERT INTO aliquot_allocation (aliquot_id, status)
      VALUES (id, reserved);
     RETURN NEXT id;
   END LOOP;
   RETURN;
 END;
' LANGUAGE 'plpgsql';

-- remove_id
-- INPUT: none
-- OUTPT: none
-- SIDE EFFECTS: corresponding entry in aliquot_allocation table removed
--
-- must be set as a trigger before insert on aliquot

CREATE OR REPLACE FUNCTION remove_id() RETURNS TRIGGER AS '

 BEGIN
  DELETE FROM aliquot_allocation
  WHERE aliquot_id = NEW.aliquot_id;
  RETURN NEW;
 END;
' LANGUAGE 'plpgsql';


CREATE TRIGGER remove_id_from_pool
    BEFORE INSERT ON aliquot
    FOR EACH ROW
    EXECUTE PROCEDURE remove_id ();

