# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: query.py 749 2005-06-28 22:21:49Z sidnei $
"""

from persistent import Persistent
from zope.interface import implements
from zope.app.utility.interfaces import ILocalUtility
from zope.app.container.contained import Contained
from sys import modules
from sip.interfaces.utilities import IQuery
from sip.results import ResultsSet
from sqlos.interfaces import ISQLAttributeAnnotatable

def doSampleNameSort(results):
    sortlist = [(r.name,r) for r in results]
    sortlist.sort()
    results[:] = [r[1] for r in sortlist]

def doSampleIDSort(results):
    sortlist = [(r.id,r) for r in results]
    sortlist.sort()
    results[:] = [r[1] for r in sortlist]

def doDateGeneratedSort(results):
    sortlist = [(r.date_generated,r) for r in results]
    sortlist.sort()
    results[:] = [r[1] for r in sortlist]

class Query(Persistent, Contained):

    implements(IQuery, ILocalUtility, ISQLAttributeAnnotatable)

    def search(self, query):
        results = []
        ctx = self.__parent__
        for cmd in query.cmds:
            results.extend(cmd.execute(ctx))

        results = self._combine(results)
        if query.sorton is not None:
            self._sort(results, query.sorton)
        return ResultsSet(results)

    def _sort(self, results, sorton):
        method = 'do'+sorton+'Sort'
        method = getattr(modules[self.__module__], method)
        method(results)

    def _combine(self, results):
        raw_results = {}
        for result in results:
            sid = result.id
            if sid in raw_results:
                raw_results[sid].append(result)
            else:
                raw_results[sid] = []
                raw_results[sid].append(result)
        results = []
        for res in raw_results.values():
            if len(res) == 1:
                results.extend(res)
            else:
                results.append(_combine(res))
        return results

def _combine(results):
    res = results[0]
    for r in results[1:]:
        res.update(r)
    return res
