# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: splitderive.py 759 2005-06-28 23:45:42Z sidnei $
"""

from persistent import Persistent

from zope.interface import implements
from zope.app.container.contained import Contained
from zope.app import zapi
from zope.event import notify
from zope.app.event.objectevent import ObjectModifiedEvent
from zope.app.utility.interfaces import ILocalUtility

from sip.interfaces.utilities import ISplitDerive
from sip.browser import api
from sqlos.interfaces import ISQLAttributeAnnotatable

def testAliquot(rec):
    if (hasattr(rec,'depleted') or rec.has_key('depleted')) \
                                and rec['depleted']:
        return True
    weight = volume = concentration = 0.0
    if rec['weight']:
        weight = float(rec['weight'])
    if rec['volume']:
        volume = float(rec['volume'])
    if rec['concentration']:
        concentration = float(rec['concentration'])
    if (weight + concentration + volume) or rec['aliquot_count']:
        return True
    return False

class SplitDerive(Persistent, Contained):
    implements(ISplitDerive, ILocalUtility, ISQLAttributeAnnotatable)

    def derive(self, sample_id, aliquot_ids):
        factory = api.factory('sip.OriginSampleAliquots', context=self)
        for id in aliquot_ids:
            kw = {}
            kw['aliquot_id'] = id
            kw['sample_id'] = sample_id
            factory(**kw)

    def split(self, base_aliquots, new_aliquots):
        factory = api.factory('sip.Aliquot', context=self)
        aliquots = []
        for rec in base_aliquots:
            rec = dict(rec)
            id = int(rec['id'])
            del rec['id']
            aliquot = factory.get(id)
            if rec.has_key('depleted') and rec['depleted']:
                rec['depleted'] = True
            for k in ('weight', 'volume', 'concentration'):
                if rec[k]:
                    rec[k] = float(rec[k])
                else:
                    rec[k] = None
                unit = k + '_unit'
                if rec.has_key(unit) and rec[unit]:
                    rec[k + '_id'] = int(rec[unit])
                    del rec[unit]
                elif not rec.has_key(k + '_id'):
                    rec[k + '_id'] = None
            for k in ('container_id', 'aliquot_count'):
                if rec[k]:
                    rec[k] = int(rec[k])
                else:
                    rec[k] = None

            if rec.has_key('is_low') and rec['is_low']:
                rec['is_low'] = True
            unchanged = True
            for k in rec.keys():
                if (getattr(aliquot, k)!=rec.get(k) and k!='commit_message'):
                    unchanged = False
                    break
            if not unchanged:
                values = {}
                for k in ('concentration', 'volume', 'weight'):
                    value = rec[k]
                    if value is not None:
                        values[k] = value
                        rec[k] = None
                aliquot.set(**rec)
                notify(ObjectModifiedEvent(aliquot))
                if values:
                    for k,v in values.items():
                        setattr(aliquot, k , v)
                    notify(ObjectModifiedEvent(aliquot))
            aliquots.append(aliquot)
            for rec in new_aliquots:
                rec = dict(rec)
                if int(rec['base_id']) == id:
                    if not testAliquot(rec):
                        continue
                    kw = {}
                    kw['id'] = int(rec['id'])
                    kw['sample_id'] = aliquot.sample_id
                    kw['project_id'] = aliquot.project_id
                    for k in ('weight', 'volume', 'concentration'):
                        if rec[k]:
                            kw[k] = float(rec[k])
                        else:
                            kw[k] = None
                        unit = k + '_unit'
                        if rec.has_key(unit) and rec[unit]:
                            kw[k + '_id'] = int(rec[unit])
                        elif rec.has_key(k + '_id'):
                            kw[k + '_id'] = rec[k + '_id']
                        else:
                            kw[k + '_id'] = None
                    kw['commit_message'] = 'From Split'
                    if rec['container_id']:
                        kw['container_id'] = int(rec['container_id'])
                    else:
                        kw['container_id'] = None
                    count = rec['aliquot_count']
                    if count:
                        kw['aliquot_count'] = count
                    kw['comment'] = rec['comment']
                    values = {}
                    for k in ('concentration', 'volume', 'weight'):
                        value = kw[k]
                        if value:
                            values[k] = value
                            kw[k] = None
                    new_aliquot = factory(**kw)
                    if values:
                        for k,v in values.items():
                            setattr(new_aliquot, k, v)
                        notify(ObjectModifiedEvent(new_aliquot))
                    aliquots.append(new_aliquot)
        return aliquots
