# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: utils.py 759 2005-06-28 23:45:42Z sidnei $
"""

__metaclass__ = type

from zope.app import zapi
from zope.app.container.interfaces import IAdding
from zope.security.management import queryInteraction
from zope.app.datetimeutils import parseDatetimetz
from zope.schema.interfaces import ValidationError
from sip.i18n import _

class UniqueNamesError(ValidationError):
    __doc__ = _("""This value must be unique""")

def getPrincipals():
    principals = []
    interaction = queryInteraction()
    if interaction is not None:
        for participation in interaction.participations:
            principals.append(participation.principal.id)
    return principals

class Unique:

    def __init__(self, className, columnName, limit=0, byProject=False):
        self.className = className
        self.columnName = columnName
        self.limit = limit
        self.byProject = byProject

    def __call__(self, field, value):
        from sqlobject.converters import sqlrepr
        from sip.browser import api
        limit = self.limit
        context = field.context
        if not IAdding.providedBy(field.context):
            limit += 1
        else:
            context = context.context
        if self.byProject:
            project = api.getProject(context)
        context = api.factory(self.className, context=context)
        query = "%s = %s" % (getattr(context.q, self.columnName),
                             sqlrepr(value, 'postgres'))
        if self.byProject:
            query += ' and project_id = %d' % project.id
        items = context.select(query)
        if items.count() > limit:
            raise UniqueNamesError
        return value

class PasswordMatch:

    #def __init__(self, className, columnName):
        #self.className = className
        #self.columnName = columnName

    def __call__(self, field, value):
        form = field.context.request.form
        if value == form['field.password']:
            return value
        return False
