# Copyright (c) 2004 by RUNYAGA, LLC.
# http://www.runyaga.com/

"""
$Id: vocabulary.py 758 2005-06-28 23:02:39Z sidnei $
"""

from zope.app import zapi
from zope.schema.interfaces import IField
from sip.browser import api
from sqlos.utils import fromItems

def unique(l):
    d = {}
    for index, tup in enumerate(l):
        if tup[0] in d:
            tup = ("%s (%s)" % tup, tup[1])
            l[index] = tup
        else:
            d[tup[0]] = None
    return l

def sort_by_name(a, b):
    return cmp(a[0].lower(), b[0].lower())

def LocationsVocab(context):
    avail_locations = api.getRoot(context)['locations'].values()
    items = [(x.name, x.id) for x in avail_locations]
    items.sort(sort_by_name)
    return fromItems(items, context)

def TechniciansVocab(context):
    avail_technicians = api.getRoot(context)['technicians'].values()
    items = [(x.username, x.id) for x in avail_technicians]
    items.sort(sort_by_name)
    return fromItems(items, context)

def SampleTypesVocab(context):
    stypes = api.getRoot(context)['sampletypes'].values()
    items = [(x.name, x.id) for x in stypes]
    items.sort(sort_by_name)
    return fromItems(items, context)

def SubjectTypesVocab(context):
    stypes = api.getRoot(context)['subjecttypes'].values()
    items = [(x.name, x.id) for x in stypes]
    items.sort(sort_by_name)
    return fromItems(items, context)

def AvailProjectsVocab(context):
    avail_projects = api.getRoot(context)['projects'].items()
    items = [(x.name, id) for id, x in avail_projects]
    return fromItems(items, context)

def AvailPrincipalsVocab(context):
    avail_principals = api.getRoot(context)['principals'].items()
    items = [(x.name, id) for id, x in avail_principals]
    items.sort(sort_by_name)
    return fromItems(items, context)

def AvailContainerTypesVocab(context):
    avail_containertypes = api.getRoot(context)['containertypes'].items()
    items = [(x.name, id) for id, x in avail_containertypes]
    items.sort(sort_by_name)
    return fromItems(items, context)

def AvailConcentrationUnitsVocab(context):
    items = api.getRoot(context)['concentrationunits'].items()
    items = [(x.label, x.id) for id, x in items]
    return fromItems(items, context)

def AvailWeightUnitsVocab(context):
    items = api.getRoot(context)['weightunits'].items()
    items = [(x.label, x.id) for id, x in items]
    return fromItems(items, context)

def AvailVolumeUnitsVocab(context):
    items = api.getRoot(context)['volumeunits'].items()
    items = [(x.label, x.id) for id, x in items]
    return fromItems(items, context)

def ProjectTechniciansVocab(context):
    project = api.getProject(context)
    avail_technicians = zapi.traverse(project, 'technicians').values()
    items = [(x.username, x.id) for x in avail_technicians]
    items.sort(sort_by_name)
    return fromItems(items, context)

def ProjectSampleTypesVocab(context):
    project = api.getProject(context)
    avail_sample_types = zapi.traverse(project, 'sampletypes').values()
    items = [(x.name, x.id) for x in avail_sample_types]
    items.sort(sort_by_name)
    return fromItems(items, context)

def ContainerTypesVocab(context):
    types = api.getRoot(context)['containertypes'].values()
    gp = zapi.getParent
    if api.queryContainer(gp(context)) is not None:
        types = filter(lambda x: not x.root, types)
    else:
        types = filter(lambda x: x.root, types)
    items = [(p.name, p.id) for p in types]
    items.sort(sort_by_name)
    return fromItems(items, context)

class VocabularyProxy:

    def __init__(self, callable):
        self.callable = callable

    def __call__(self, context):
        return self.callable

# This one is tricky, because it needs to get filtered. I can't think
# of a simpler implementation What we are doing here is to have a
# proxy that returns the callable function, which is then called on
# bind() with the context *and* the field.
def _FieldTypesVocab(context, field):
    factories = zapi.getFactoriesFor(IField, context=context)
    items = [(factory.title, name) for name, factory in factories]
    items = filter(field.filterfields, items)
    items.sort()
    return fromItems(items, context)

FieldTypesVocab = VocabularyProxy(_FieldTypesVocab)

def CarriersVocab(context):
    carriers = api.getRoot(context)['carriers'].values()
    items = [(x.name, x.id) for x in carriers]
    items.sort(sort_by_name)
    return fromItems(items, context)

def FundingSourcesVocab(context):
    factory = api.factory('sip.FundingSource', context=context)
    results = factory.select(orderBy='funding_source_id')
    items = [(x.name, x.id) for x in results]
    return fromItems(items, context)

def TempMethodVocab(context):
    factory = api.factory('sip.TempMethod', context=context)
    results = factory.select(orderBy='temp_method_id')
    items = [(x.name, x.id) for x in results]
    return fromItems(items, context)
