##############################################################################
#
# Copyright (c) 2004 Enfold Systems LLC. All rights reserved.
#
# This software is distributed under the terms of the Zope Public
# License (ZPL) v2.1. See COPYING.txt for more information.
#
##############################################################################
"""
$Id: __init__.py 6801 2004-09-29 23:30:04Z dreamcatcher $
"""

__metaclass__ = type

from datetime import datetime, date
from zope.interface import classImplements
from zope.app import zapi
from zope.security.checker import NamesChecker, NoProxy, defineChecker

from sqlobject.dbconnection import DBConnection, DBAPI
from sqlobject import _mysql, _postgres, _sybase
from sqlobject.main import SQLObject, SelectResults
from sqlobject.sqlbuilder import registerConverter, \
     SQLObjectTable
from sqlos.interfaces import IDBConnection, IDBAPI, ISQLConnection
from sqlos.interfaces import IISQLObject, ISQLObject, ISelectResults

defineChecker(SQLObjectTable, NoProxy)

classImplements(DBConnection, IDBConnection)
classImplements(DBAPI, IDBAPI)
classImplements(_mysql.builder(), ISQLConnection)
classImplements(_postgres.builder(), ISQLConnection)
classImplements(_sybase.builder(), ISQLConnection)
classImplements(SQLObject, ISQLObject)
classImplements(SelectResults, ISelectResults)

from _sqlos import SQLOS

def caller(n=1):
    """Return the name of the calling function n levels up in the frame stack.
    Ex: caller(0) ==> 'caller'; def f(): return caller(); f() ==> 'f'
    """
    import inspect
    return inspect.getouterframes(inspect.currentframe())[n][3]


def DateTimeConverter(value, db=None):
    return repr(value.isoformat())

registerConverter(datetime, DateTimeConverter)

def DateConverter(value, db=None):
    if isinstance(value, datetime):
        return repr(value.date().isoformat())
    return repr(value.isoformat())

registerConverter(date, DateConverter)


def getFactory(name, context=None, default=None):
    return zapi.queryUtility(IISQLObject, name, default=default, context=context)
