##############################################################################
#
# Copyright (c) 2004 Enfold Systems LLC. All rights reserved.
#
# This software is distributed under the terms of the Zope Public
# License (ZPL) v2.1. See COPYING.txt for more information.
#
##############################################################################
"""
$Id: __init__.py 5216 2004-06-21 18:33:07Z dreamcatcher $
"""

from zope.interface import implements, directlyProvides
from sqlobject.main import SQLObject
from sqlos.connection import ConnectionDescriptor
from sqlos.interfaces import ISQLObject, IISQLObject

class SQLOS(SQLObject):
    """Subclass SQLObject to enable ``lazy updates`` by default,
    as well as adding knowledge to register ``dirty`` objects
    with SQLObjectTransactionManager so they get sync'd on transaction
    boundaries.
    """
    implements(ISQLObject)

    _connection = ConnectionDescriptor()
    _lazyUpdate = True

    def _set_dirty(self, value):
        if value:
            self._connection._dm.register(self)
        else:
            # XXX: 'objects' shouldn't really be fiddled directly like this.
            # Probably there should be an unregister function.
            #   - Andrew Bennetts, 2005-01-05
            self._connection._dm.objects.discard(self)
        self._dirty = value

    def _get_dirty(self):
        return self._dirty

    dirty = property(_get_dirty, _set_dirty)

    def get(cls, id, connection=None, selectResults=None):
        # While interacting with zope, we may end up having
        # objects in the cache that have a __parent__ set.
        # This may be confusing when expect to get a object
        # which has no __parent__ and thats not what you get.
        val = super(SQLOS, cls).get(id, connection=connection,
                                    selectResults=selectResults)
        if getattr(val, '__parent__', None) is not None:
            val.__parent__ = None
            val.__name__ = None
        return val
    get = classmethod(get)

    def __repr__(self):
        return '<%s at 0x%x>' % (self.__class__.__name__, id(self))

    def setConnection(cls, connection):
        if connection is not None:
            cls._connection = connection

    setConnection = classmethod(setConnection)
