##############################################################################
#
# Copyright (c) 2004 Enfold Systems LLC. All rights reserved.
#
# This software is distributed under the terms of the Zope Public
# License (ZPL) v2.1. See COPYING.txt for more information.
#
##############################################################################
"""
$Id: adapter.py 12393 2005-05-16 22:06:40Z dreamcatcher $
"""

__metaclass__ = type

from transaction import get_transaction

from sqlobject.dbconnection import DBAPI
from sqlobject import _mysql, _postgres, _sybase, _sqlite
from sqlos.interfaces import ISQLObject
from sqlos.transaction import SQLObjectTransactionManager

from zope.app.rdb.interfaces import DatabaseException
from zope.app.container.interfaces import INameChooser


class SQLObjectNameChooser:

    def __init__(self, context):
        self.context = context

    def checkName(self, name, object):
        return True

    def chooseName(self, name, object):
        if ISQLObject.providedBy(object):
            package = object.__module__.split('.')[0]
            return "%s.%s.%s" % (package, object.__class__.__name__, object.id)
        return None

class ConnectionAdapter:

    def __init__(self, connection):
        DBAPI.__init__(self)
        self._connection = connection
        self.autoCommit = None
        self.debug = 0
        self.supportTransactions = False
        self._dm = SQLObjectTransactionManager(connection)
        get_transaction().join(self._dm)

    def makeConnection(self):
        return self._connection

    def _runWithConnection(self, meth, *args):
        conn = self.getConnection()
        try:
            try:
                val = meth(conn, *args)
            except Exception, exc:
                raise DatabaseException, tuple(exc.args)
        finally:
            self.releaseConnection(conn)
        return val

class MySQLAdapter(ConnectionAdapter, _mysql.builder()): pass
class PostgresAdapter(ConnectionAdapter, _postgres.builder()): pass
class SybaseAdapter(ConnectionAdapter, _sybase.builder()): pass
class SQLiteAdapter(ConnectionAdapter, _sqlite.builder()): pass
