##############################################################################
#
# Copyright (c) 2004 Enfold Systems LLC. All rights reserved.
#
# This software is distributed under the terms of the Zope Public
# License (ZPL) v2.1. See COPYING.txt for more information.
#
##############################################################################
"""
$Id: adapter.py 5212 2004-06-21 18:09:05Z philikon $
"""
__metaclass__ = type

from zope.interface import implements

from zope.app import zapi
from zope.app.pluggableauth.interfaces import ILoginPasswordPrincipalSource
from zope.app.pluggableauth import SimplePrincipal
from zope.exceptions import NotFoundError

from sqlos.container import SQLObjectContainer
from sqlos.interfaces.auth import ISQLObjectPrincipalSource
from sqlos.interfaces.auth import IPrincipalContainer

class SQLObjectPrincipalSource(SQLObjectContainer):
    """A Principal Source for SQLObject-based data."""

    implements(ILoginPasswordPrincipalSource,
               ISQLObjectPrincipalSource,
               IPrincipalContainer)

    def __init__(self):
        pass

    def readPrincipals(self):
        principals = []
        for user in self.values():
            userinfo = (user.username, user.password, user.fullname)
            p = SimplePrincipal(*userinfo)
            p.__parent__ = self
            p.__name__ = p.login
            p.id = "%s.%s" % (p.login, user.id)
            principals.append(p)
        return principals

    def getPrincipal(self, id):
        """See IPrincipalSource.

        In this case the login equals the id.
        """
        for p in self.readPrincipals():
            if p.id == id:
                return p
        raise NotFoundError, id

    def getPrincipals(self, name):
        """See IPrincipalSource."""
        return filter(lambda p: p.login.find(name) != -1,
                      self.readPrincipals())

    def authenticate(self, login, password):
        """See ILoginPasswordPrincipalSource. """
        for user in self.readPrincipals():
            if user.login == login and user.password == password:
                return user
