##############################################################################
#
# Copyright (c) 2004 Enfold Systems LLC. All rights reserved.
#
# This software is distributed under the terms of the Zope Public
# License (ZPL) v2.1. See COPYING.txt for more information.
#
##############################################################################
"""
$Id: factory.py 5216 2004-06-21 18:33:07Z dreamcatcher $
"""
__metaclass__ = type

from transaction import get_transaction
from zope.interface import implements
from zope.component import ComponentLookupError
from zope.app import zapi
from zope.app.rdb.interfaces import IZopeDatabaseAdapter
from sqlos.interfaces import IZopeSQLConnection, IConnectionName
from sqlos.adapter import PostgresAdapter
import warnings

class SQLObjectWarning(UserWarning): pass

class ConnectionDescriptor:

    def __init__(self, name=None):
        self.name = name

    def __get__(self, inst, cls=None):
        context = inst is None and cls or inst
        name = self.name
        if name is None:
            try:
                ut = zapi.getUtility(IConnectionName, context=context)
            except ComponentLookupError:
                return self
            if ut is None:
                warnings.warn("Couldn't find ISQLConnectionName utility. "
                              "Please verify your setup.",
                              SQLObjectWarning, 2)
                return
            name = ut.name
        try:
            return getConnection(context, name)
        except ComponentLookupError:
            return self

    def __set__(self, inst, value):
        # Ignore, so we don't get overriden.
        # We always use the connections from the
        # global thread cache.
        pass

# Connection cache, one per thread, a-la Transaction.
# This code was heavily based on ZODB.Transaction
connCache = {}

try:
    import thread
except:
    def getConnection(context, name):
        global connCache
        if not connCache.get(name):
            newconn = zapi.queryUtility(IZopeDatabaseAdapter, name,
                                        default=None,
                                        context=context)
            if newconn is None:
                warnings.warn("Couldn't find a rdb connection by the "
                              "name %s. Please verify your setup." % name,
                              SQLObjectWarning, 3)
            connCache[name] = IZopeSQLConnection(newconn())
        return connCache[name]

    def releaseConnection(name):
        global connCache
        if connCache.has_key(name):
            connCache[name].close()
            del connCache[name]
else:
    def getConnection(context, name):
        global connCache
        tid = thread.get_ident()
        key = (tid, name)
        if not connCache.get(key):
            newconn = zapi.queryUtility(IZopeDatabaseAdapter, name,
                                        default=None,
                                        context=context)
            if newconn is None:
                warnings.warn("Couldn't find a rdb connection by the "
                              "name %s. Please verify your setup." % name,
                              SQLObjectWarning, 3)
            connCache[key] = IZopeSQLConnection(newconn()).transaction()
        return connCache[key]

    def releaseConnection(name):
        global connCache
        tid = thread.get_ident()
        key = (tid, name)
        if connCache.has_key(key):
            connCache[key].close()
            del connCache[key]
