##############################################################################
#
# Copyright (c) 2004 Enfold Systems LLC. All rights reserved.
#
# This software is distributed under the terms of the Zope Public
# License (ZPL) v2.1. See COPYING.txt for more information.
#
##############################################################################
"""
$Id: test_transaction.py 5477 2004-07-07 16:06:13Z dreamcatcher $
"""
import unittest
from transaction import get_transaction

from zope.app.tests.functional import BrowserTestCase
from zope.app import zapi
from zope.app.tests import setup
from zope.app.rdb import ZopeConnection
from zope.app.rdb.interfaces import IZopeDatabaseAdapter
from zope.security.checker import defineChecker, NoProxy

from sqlos import getFactory
from sqlos.container import SQLObjectContainer
from sqlos.tests.sampleperson import SamplePerson

from psycopgda.adapter import PsycopgAdapter

__metaclass__ = type

DSN = 'dbi://zope3:123@localhost:5432/zope3'

defineChecker(SamplePerson, NoProxy)

class TestTransaction(BrowserTestCase):

    def setUp(self):
        BrowserTestCase.setUp(self)
        self.conn = PsycopgAdapter(DSN)
        utilities = zapi.getService(None, zapi.servicenames.Utilities)
        utilities.provideUtility(IZopeDatabaseAdapter, self.conn, 'stub')
        container = SQLObjectContainer()
        container = self.createContainer(container)

        # Create a service manager at the approot level
        mgr = setup.createServiceManager(container)

        container.classNames = ['SamplePerson',]
        kwargs = {'fullname':'Sidnei da Silva',
                'username':'sidnei',
                'password':'test'}
        self.person = self.createObject(container, **kwargs)
        get_transaction().commit()

    def createContainer(self, container):
        folder = self.getRootFolder()
        folder['sqlcontainer'] = container
        return folder['sqlcontainer']

    def createObject(self, container, **kwargs):
        factory = getFactory(container.classNames[0])
        person = factory.new(**kwargs)
        container['sampleperson'] =  person
        return container['SamplePerson.%s' % person.id]

    def testChange(self):
        person = self.person
        self.assertEqual(person.fullname, 'Sidnei da Silva')
        self.assertEqual(person.username, 'sidnei')
        self.assertEqual(person.password, 'test')
        person.fullname = 'Sidnei Silva'
        person.username = 'dreamcatcher'
        person.password = 'pass'
        self.assertEqual(person.fullname, 'Sidnei Silva')
        self.assertEqual(person.username, 'dreamcatcher')
        self.assertEqual(person.password, 'pass')

    def testAbort(self):
        person = self.person
        self.assertEqual(person.fullname, 'Sidnei da Silva')
        self.assertEqual(person.username, 'sidnei')
        self.assertEqual(person.password, 'test')
        person.fullname = 'Sidnei Silva'
        person.username = 'dreamcatcher'
        person.password = 'pass'
        get_transaction().abort()
        self.assertEqual(person.fullname, 'Sidnei da Silva')
        self.assertEqual(person.username, 'sidnei')
        self.assertEqual(person.password, 'test')

    def tearDown(self):
        self.person.destroySelf()
        BrowserTestCase.tearDown(self)

def test_suite():
    suite = unittest.TestSuite()
    # XXX disable for now, this is too broken
    # suite.addTest(unittest.makeSuite(TestTransaction))
    return suite


if __name__ == '__main__':
    unittest.main()
