##############################################################################
#
# Copyright (c) 2004 Enfold Systems LLC. All rights reserved.
#
# This software is distributed under the terms of the Zope Public
# License (ZPL) v2.1. See COPYING.txt for more information.
#
##############################################################################
"""
$Id: __init__.py 5216 2004-06-21 18:33:07Z dreamcatcher $
"""

import sys
import warnings

from zope.interface.verify import verifyClass
from zope.interface import directlyProvides, implements
from zope.app.component.metaconfigure import factory, utility
from zope.app.component.metaconfigure import PublicPermission, proxify
from zope.security.checker import defineChecker, Checker
from zope.security.checker import getCheckerForInstancesOf, MultiChecker
from zope.component.factory import Factory, IFactory

import sqlos
from sqlos.interfaces import IISQLObject
from sqlos.interfaces import IReadSQLObjectClass, IWriteSQLObjectClass
from sqlos.interfaces import IConnectionName

from sqlobject.main import MetaSQLObject

class SQLObjectFactory(Factory):
    implements(IFactory)

def handler(_context, component, id, title=None, description=None):

    if isinstance(component, basestring):
        component = _context.resolve(component)

    # If the class doesn't mix-in SQLOS, we do it for you, so that you
    # can use pure SQLObject classes, and they behave accordingly when
    # inside Zope3.
    # However, we issue a warning as this may have unknown side-effects.
    if not issubclass(component, sqlos.SQLOS):
        m_name = component.__module__
        c_name = component.__name__
        c_dict = dict(vars(component))
        warnings.warn("You forgot to subclass sqlos.SQLOS on "
                      "the registered factory %s.%s. We are "
                      "going to try to fix this, but this has "
                      "unknown side-effects and may break in the future. "
                      "Please fix this as soon as possible." %
                      (m_name, c_name),
                      Warning, 1)
        # XXX instead of creating a new class, we could try to modify
        # the existingclass inplace by injecting our overriden stuff
        # on SQLOS into the component __dict__.
        component = MetaSQLObject(component.__name__,
                                  (sqlos.SQLOS, component),
                                  c_dict)
        # **Warning**: Monkeypatch ahead.
        module = sys.modules[m_name]
        setattr(module, c_name, component)

    factoryObj = SQLObjectFactory(component, title, description)

    # XXX We compute a permission name from the package top-level name
    # plus the component (which should be a class) __name__ The
    # permission should probably be part of the directive here.
    package = component.__module__.split('.')[0]
    add_perm = '%s.Add%s' % (package, component.__name__)
    utility(_context, IFactory, factoryObj,
            permission=add_perm, name=id)

    if not IISQLObject.providedBy(component):
        if verifyClass(IISQLObject, component, tentative=1):
            directlyProvides(component, IISQLObject)

    # XXX We are really creating a proxy for the *class* here.
    # However, we don't want to modify the class, and we don't want
    # this to conflict with the checker created by the <content>
    # directive, so we create a Proxy using a checker with computed
    # permission names.  The permissions should probably be part of
    # the directive.

    checker = getCheckerForInstancesOf(component)
    if checker is None:
        read_perm = '%s.View%s' % (package, component.__name__)
        change_perm = '%s.Change%s' % (package, component.__name__)
        checker = MultiChecker(((IReadSQLObjectClass, read_perm),
                                (IWriteSQLObjectClass, change_perm),
                                ))
        checker = Checker(checker.get_permissions, {})
        defineChecker(component, checker)

    proxied = proxify(component, checker)

    # XXX We pass permission=None here so that the utility handler
    # doesn't override what we've just did above.
    utility(_context, IISQLObject, proxied,
            permission=None, name=id)

class ConnectionName:
    """A connection name utility"""

    implements(IConnectionName)

    def __init__(self, name):
        self.name = name

def connectionName(_context, name):
    component = ConnectionName(name)
    utility(_context, IConnectionName, component,
            permission=PublicPermission, name='')
