# spats  |  Copyright(C), 2004-5, Enfold Systems, LLC
# 
# 
# Enfold Systems, LLC
# 4500 Montrose Blvd., Suite D
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com
# 
# 
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

from SimpleHTTPServer import SimpleHTTPRequestHandler
from SocketServer import ThreadingMixIn
from BaseHTTPServer import HTTPServer
from simpletal import simpleTAL, simpleTALES
from tempfile import mkstemp

import threading
import os
import sys
import traceback
import imp
import cgi
import time
import urlparse
import socket
import traceback
import errno
import logging

from exceptions import Exception
class Unauthorized(Exception): pass

from StringIO import StringIO

from formats import getFormats, getOrder

logger = logging.getLogger("spats")

dd = os.path.dirname(os.path.abspath(__file__))
# If not with source-code, use sys.prefix, which is where distutils puts them.
if not os.path.isdir(os.path.join(dd, "base")):
    dd = os.path.join(sys.prefix, 'spats')

_config = {
    # is anonymous access allowed
    "anon_access":True,
    # ip addresses allowed access
    # not actually implemented yet
    "ip_addresses":["127.0.0.1",],
    # auth_methods
    # not actually implemented yet
    "auth_methods":["pwd_file",],
    # the root html or page template directory, everything is served out of here
    "html_dir":["/html",],
    # the base directory where common css and other files can be stored
    "base_dir":"%s/base" % dd,
    # what to show when it goes wrong
    "error_pages":{
        "500":"%s/errors/500.pt" % dd, 
        "302":"%s/errors/302.pt" % dd, 
        "403":"%s/errors/403.pt" % dd,
        "404":"%s/errors/404.pt" % dd        
        },
    # what scripts will get compiled in
    "scripts_dir":["scripts",],
    # first page to show
    "default_pages":["index.html", "index.htm", "index.pt"],
    # what templates contain macros and need compiling
    "macros_templates":["base/template.pt",],
    # a chance to throw more stuff into the context here
    "extra_context":None,
    # run in debug_mode, slower but refreshes scripts
    "debug_mode":True
    }

def importModule(location):
    name = os.path.basename(location).split('.')[0]
    path = os.path.dirname(location)
    paths = [path,]

    triple = imp.find_module(name, paths)

    for path in paths:
        if path not in sys.path:
            sys.path.append(path)

    mod = imp.load_module(name, *triple)

    return mod

_scripts = None
_templates = None

def parse_qs(data):
    old = cgi.parse_qs(urlparse.urlsplit(data)[3])
    new = {}
    for k, v in old.items():
        if len(v) ==  1:
            new[k] = v[0]
        else:
            new[k] = v
    return new

class SimplePageTemplateHandler(SimpleHTTPRequestHandler):
    temp_files = []
    custom_headers = {}

    def getScripts(self, context):
        """ Figure out all the scripts, we'll do this once on first request """
        global _scripts
        
        for dr in _config['scripts_dir']:
            scripts_dir = os.path.abspath(dr)
            # This code doesn't play well with classes - so all such code may
            # exist in a "lib" subdir.
            
            scripts_lib_dir = os.path.join(scripts_dir, "lib")
            if os.path.isdir(scripts_lib_dir) and scripts_lib_dir not in sys.path:
                sys.path.append(scripts_lib_dir)
                
        # cache all the imported scripted, unless we are in debug mode
        if _scripts is None or _config['debug_mode']:
            scripts = {}
            for dr in _config['scripts_dir']:
                for file in os.listdir(dr):
                    if file.endswith('.py'):
                        name = file.split('.py')[0]
                        mod = None
                        if name.startswith('.'):
                            continue
                        try:
                            mod = importModule('%s/%s' % (dr, name))
                        except ImportError, msg:
                            if _config['debug_mode']:
                                # if we are in debug-mode, just move on, don't cry about it
                                print "** Failed to import %s, %s" % (file, msg)
                                continue
                            else:
                                raise
                        if hasattr(mod, "__call__"):
                            scripts[name] = mod
            _scripts = scripts
        # add in a changing context, so each invocation gets a new context
        _n_scripts = {}
        for k, v in _scripts.items():
            # insert context
            # make key the __call__ method
            v.context = context
            v.self = self
            _n_scripts[k] = v.__call__
        return _n_scripts
    
    def getTemplates(self):
        """ Get the templates """
        global _templates
        if _templates is None or _config['debug_mode']:
            templates = {}
            for t in _config['macros_templates']:
                if t.endswith('.pt'):
                    name = t.split('/')[-1].split('.pt')[0]
                    templateFile = open(t, 'rb')
                    macros = simpleTAL.compileHTMLTemplate (templateFile)
                    templateFile.close()
                    templates[name] = macros
            _templates = templates
        return _templates
    
    def getContext(self, **kw):
        """ Figure out the context """
        # Trying to cache this does not make sense
        # since headers and options will change for each request
        context = simpleTALES.Context(allowPythonPath=1)
        context.addGlobal(
                "request", {
                    "headers":self.headers,
                    "query":parse_qs(self.path),
                    "url":self.path.split('?')[0],
                    }
        )

        context.addGlobal("macros", self.getTemplates())
        context.addGlobal("scripts", self.getScripts(context))
        if _config["extra_context"]:
            for k, v in _config["extra_context"].items():
                context.addGlobal(k, v)

        # always add in last error
        t, v, tb = sys.exc_info()
        kw["err_type"] = str(t)
        kw["err_value"] = str(v)
        strtb = StringIO()
        traceback.print_tb(tb, limit=50, file=strtb)
        kw["err_tb"] = strtb.getvalue()
        context.addGlobal("options", kw)
        return context
    
    def processPT(self, path, **kw):
        """ Process the path into a page template """
        # open up pt
        self.check_security()
        templateFile = open (path, 'rb')
        template = simpleTAL.compileHTMLTemplate (templateFile)
        templateFile.close()

        # write out to temp
        fd, out = mkstemp()
        os.close(fd)
        outfile = open(out, "wb")
        template.expand(self.getContext(**kw), outfile)
        outfile.close()
        
        return out

    def reset(self):
        self.custom_headers = {}
        self.temp_files = []

    def check_security(self):
    
        if not self._check_security:
            return
            
        if _config['anon_access']:
            return
       
        # implement ip checking
        # implement other checks
       
        raise Unauthorized
        
    def do_HEAD(self, *args, **kw):
        """ Perfom security """
        self.reset()
        SimpleHTTPRequestHandler.do_HEAD(self, *args, **kw)
    
    def do_GET(self, *args, **kw):
        """ The base class throws around files, we need to use temp files
        for page templates, this cleans them up and perform security"""
        self.reset()
        self._check_security = True
        try:
            f = self.send_head()
        except Unauthorized:
            f = self.send_error(403)
        except SystemExit:
            self.server.stop()
            return
        except:
            # eek bare except
            traceback.print_exc()
            f = self.send_error(500)
       
        # and what does this mean?
        assert(f != None)
         
        if f:
            self.copyfile(f, self.wfile)
            f.close()
        while self.temp_files:
            t = self.temp_files.pop()
            try:
                os.remove(t)
            except OSError, why:
                # let the os clean it then
                logger.warning("Failed to remove temp file '%s': %s", t, why)

    # send base class logging to a real logger.
    def log_request(self, code='-', size='-'):
        logger.debug('"%s" %s %s', self.requestline, str(code), str(size))
    
    def log_message(self, format, *args):
        logger.info(format, *args)
    
    def log_error(self, format, *args):
        logger.error(format, *args)
    
    def send_error(self, error, message=None):
        errors = _config["error_pages"]
        if str(error) in errors.keys():
            self._check_security = False
            return self.send_head(
                path=errors[str(error)],
                status_code=error,
                )
        raise ValueError, "Unknown error code: %s"  % error
    
    def add_header(self, k, v):
        """ Add in a header """
        hs = self.custom_headers.get(k, [])
        hs.append(v)
        self.custom_headers[k] = hs
    
    def send_head(self, path=None, status_code=200):
        """Common code for GET and HEAD commands.

        This sends the response code and MIME headers.

        Return value is either a file object (which has to be copied
        to the outputfile by the caller unless the command was HEAD,
        and must be closed by the caller under all circumstances), or
        None, in which case the caller has nothing further to do.

        """
        # allow path to be forced
        if path is None:
            path = self.path
            
            assert path.find('..') < 0, "You cannot pass .. in the URL"
            
            # look through all the html directories
            for dr in _config['html_dir']:
                p = path
                if not p.startswith('%s/' % dr):
                    p = dr + p
                p = p.split('?')[0]
                if os.path.exists(p):
                    path = p
                    break
                    
            # finally look in base if not there
            if not os.path.exists(path):
                path = _config['base_dir'] + self.path
                path = path.split('?')[0]
        
        f = None
        if os.path.isdir(path):
            for index in _config['default_pages']:
                index = os.path.join(path, index)
                if os.path.exists(index):
                    path = index
                    break
                    
        if path.endswith('.pt') or path.endswith('.ptx'):
            # do we check security, before processing this
            fpath = self.processPT(path)
            # register temp file
            self.temp_files.append(fpath)
            ctype = 'text/html'
            if path.endswith('.ptx'):
                ctype = 'text/xml'
        else:
            fpath = path
            ctype = self.guess_type(path)

        # If mode != 'rb', then we can't use stat to get the content-length
        # (as size on disk may not == number of \n translated bytes).
        # For now let the client deal with \n issues.
        mode = 'rb'
        try:
            f = open(fpath, mode)
        except IOError:
            raise
            return self.send_error(404)
            
        # write out the headers
        self.send_response(status_code)
        self.send_header("Content-type", ctype)
        self.send_header("Content-Length", str(os.fstat(f.fileno())[6])) 
        
        # allow a script to add in custom headers
        if self.custom_headers:
            for k, v in self.custom_headers.items():
                if isinstance(v, list):
                    for r in v:
                        self.send_header(k, r)
                else:
                    self.send_header(k, v)
        self.end_headers()

        # what does this mean?
        assert f != None
        return f
        
    def list_directory(self, path):
        """ Override directory listings in higher class """
        self.send_error(403, "You are not allowed to list the contents of this directory.")        

class SpatsHTTPServer(HTTPServer):
    def __init__(self, *args):
        # assume we will start, and prevent issues with
        # the main thread checking before it is set.
        self.running = True
        HTTPServer.__init__(self, *args)

    def serve(self):
        # Run until stop() is called.
        self.running = True
        try:
            try:
                # Re rely on a timeout to prevent this blocking.
                while self.running:
                    self.handle_request()
            except SystemExit:
                pass
            except:
                logger.exception("Unexpected exception in spats server")
        finally:
            self.running = False

    def stop(self):
        # later we may need to do something fancier.
        self.running = False

# find a free port 
def freePort(server, port):
    # 5 seconds should be enough to get a response
    # from most local servers
    socket.setdefaulttimeout(1)
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    try:
        s.connect((server, port))
    except socket.error, exc:
        # Be careful not to mask real errors (such as invalid server string!)
        # didn't work for me, we want a time out right?
        try:
            if exc[0] in (errno.WSAECONNREFUSED, errno.WSAETIMEDOUT):
                raise
        except AttributeError:
            # for people where WSAECONNREFUSED doesn't work
            pass
        s.close()
        return 1
    return 0


def start(server_address=None, config=None, format=None):
    # set defaults
    
    if server_address is None:
        server_address = ('localhost', 8080)

    if format is None:
        # default order set in formats...
        format = getOrder()

    # _config is a global
    # only override existing configs
    if config:
        for k, v in config.items():
            if _config.has_key(k):
                _config[k] = v
    
    server, port = server_address
    # give this 50 attempts
    for x in range(0, 50):
        res = freePort(server, port)
        if res:
            break
        # note we cumulative add for the hell of it
        port += x

    server_address = server, port
    logger.debug("Local server address is %s", server_address)

    httpd = SpatsHTTPServer(server_address, SimplePageTemplateHandler,)

    t = threading.Thread(target = httpd.serve)
    t.setDaemon(True)
    t.start()

    # call each of the formats in the requested order
    # if something dies, yell
    try:
        formats = getFormats()
        for fmt in format:
            if fmt in formats: # if format available...
                try:
                    formats[fmt].__call__(server_address, httpd)
                    break
                except:
                    print "** Calling %s failed" % fmt
                    if config["debug_mode"]:
                        raise
    finally:
        httpd.stop()
        t.join(5)
        if t.isAlive():
            logger.warning("Server thread did not stop")
    
if __name__=='__main__':
    pass
